"""
Cryptsetup - combine metadata about LUKS devices
================================================

Combine outputs of LuksDump and LuksMeta parsers (with the same UUID) into a
single dictionary.
"""

import copy

from insights import SkipComponent
from insights.core.plugins import combiner
from insights.parsers.cryptsetup_luksDump import LuksDump
from insights.parsers.luksmeta import LuksMeta


@combiner(LuksDump, optional=[LuksMeta])
class LuksDevices(list):
    """
    Combiner for LUKS encrypted devices information. It uses the results of
    the ``LuksDump`` and ``LuksMeta`` parser (they are matched based UUID of
    the device they were collected from).


    Examples:
        >>> luks_devices[0]["header"]["Version"]
        '1'
        >>> "luksmeta" in luks_devices[0]
        True
        >>> "luksmeta" in luks_devices[1]
        False
        >>> luks_devices[0]["luksmeta"][0]
        Keyslot on index 0 is 'active' with no embedded metadata
    """

    def __init__(self, luks_dumps, luks_metas):
        luksmeta_by_uuid = {}

        if luks_metas:
            for luks_meta in luks_metas:
                if "device_uuid" not in luks_meta:
                    continue

                luksmeta_by_uuid[luks_meta["device_uuid"].lower()] = luks_meta

        for luks_dump in luks_dumps:
            uuid = luks_dump.dump["header"]["UUID"].lower()
            luks_dump_copy = copy.deepcopy(luks_dump.dump)

            if luks_metas and uuid in luksmeta_by_uuid:
                luks_dump_copy["luksmeta"] = luksmeta_by_uuid[uuid]

            self.append(luks_dump_copy)

        if not self:
            raise SkipComponent



# -*- coding: utf-8 -*-
"""
Utilities for POSIX Bracket Expressions
=======================================

Per the Insights Client Configuration Guide, the "POSIX Bracket Expressions",
is also supported by the "file-content-redaction.yaml":
-  https://access.redhat.com/documentation/en-us/red_hat_insights/2023/html/client_configuration_guide_for_red_hat_insights/con-insights-client-data-redaction_insights-cg-data-redaction#proc-insights-client-cg-redact-pattern-keyword-yaml_insights-cg-data-redaction


This module provides a method to convert the "POSIX Bracket Expressions" to
ASCII Regular Expressions.


Reference:
- https://www.regular-expressions.info/posixbrackets.html
"""

POSIX_ASCII = {
    "[[:alnum:]]": "[a-zA-Z0-9]",
    "[[:alpha:]]": "[a-zA-Z]",
    "[[:ascii:]]": r"[\x00-\x7F]",
    "[[:blank:]]": r"[ \t]",
    "[[:cntrl:]]": r"[\x00-\x1F\x7F]",
    "[[:digit:]]": "[0-9]",
    "[[:graph:]]": r"[\x21-\x7E]",
    "[[:lower:]]": "[a-z]",
    "[[:print:]]": r"[\x20-\x7E]",
    "[[:punct:]]": r"""[!"\#$%&'()*+, \-./:;<=>?@\[ \\\]^_‘{|}~]""",
    "[[:space:]]": r"[ \t\r\n\v\f]",
    "[[:upper:]]": "[A-Z]",
    "[[:word:]]": "[A-Za-z0-9_]",
    "[[:xdigit:]]": "[A-Fa-f0-9]",
}


def replace_posix(text):
    """
    Replace the POSIX Bracket Expressions appear in the `text` to ASCII
    Regular Expressions.
    """
    new_text = text
    for old, new in POSIX_ASCII.items():
        new_text = new_text.replace(old, new)
    return new_text



"""
NeutronL3AgentLog - file ``/var/log/neutron/l3-agent.log``
==========================================================
"""

from .. import LogFileOutput, parser
from insights.specs import Specs


@parser(Specs.neutron_l3_agent_log)
class NeutronL3AgentLog(LogFileOutput):
    """
      Parse the ``/var/log/neutron/l3-agent.log`` file.

      .. note::
          Please refer to its super-class :class:`insights.core.LogFileOutput` for more
          details.

      Sample log lines::

          2017-09-17 10:05:06.241 141544 INFO neutron.agent.l3.ha [-] Router 01d51830-0e3e-4100-a891-efd7dbc000b1 transitioned to backup
          2017-09-17 10:05:07.828 141544 WARNING neutron.agent.linux.iptables_manager [-] Duplicate iptables rule detected. This may indicate a bug in the the iptables rule generation code. Line: -A neutron-l3-agent-INPUT -p tcp -m tcp --dport 9697 -j DROP
          2017-09-17 10:05:07.829 141544 WARNING neutron.agent.linux.iptables_manager [-] Duplicate iptables rule detected. This may indicate a bug in the the iptables rule generation code. Line: -A neutron-l3-agent-INPUT -m mark --mark 0x1/0xffff -j ACCEP

      Examples:
          >>> len(agent_log.get("Duplicate iptables rule detected")) == 2
          True
          >>> from datetime import datetime
          >>> len(list(agent_log.get_after(datetime(2017, 2, 17, 10, 5, 7))))
          3

      """
    pass



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import rhsm_releasever as rhsm_releasever_module
from insights.parsers.rhsm_releasever import RhsmReleaseVer
from insights.tests import context_wrap
from insights.tests.parsers import skip_component_check

RHEL_MAJ_MIN = '{"releaseVer": "6.10"}'
RHEL_MAJ_1 = '{"releaseVer": "7Server"}'
RHEL_MAJ_2 = '{"releaseVer": "8"}'
RHEL_NONE = '{"releaseVer": ""}'
RHEL_NONE_2 = '{"releaseVer": null}'
RHEL_EMPTY = '{}'


def test_rhsm_releasever():
    relver = RhsmReleaseVer(context_wrap(RHEL_MAJ_MIN))
    assert relver['releaseVer'] == '6.10'
    assert relver.set == '6.10'
    assert relver.major == 6
    assert relver.minor == 10

    relver = RhsmReleaseVer(context_wrap(RHEL_MAJ_1))
    assert relver['releaseVer'] == '7Server'
    assert relver.set == '7Server'
    assert relver.major == 7
    assert relver.minor is None

    relver = RhsmReleaseVer(context_wrap(RHEL_MAJ_2))
    assert relver['releaseVer'] == '8'
    assert relver.set == '8'
    assert relver.major == 8
    assert relver.minor is None

    relver = RhsmReleaseVer(context_wrap(RHEL_NONE))
    assert relver['releaseVer'] == ''
    assert relver.set is None
    assert relver.major is None
    assert relver.minor is None

    relver = RhsmReleaseVer(context_wrap(RHEL_NONE_2))
    assert relver['releaseVer'] is None
    assert relver.set is None
    assert relver.major is None
    assert relver.minor is None

    with pytest.raises(SkipComponent) as e_info:
        RhsmReleaseVer(context_wrap(RHEL_EMPTY))
    assert "releaseVer is not in data" in str(e_info.value)


def test_empty():
    assert 'Empty output.' in skip_component_check(RhsmReleaseVer)


def test_doc_examples():
    env = {
        'rhsm_releasever': RhsmReleaseVer(context_wrap(RHEL_MAJ_MIN)),
    }
    failed, total = doctest.testmod(rhsm_releasever_module, globs=env)
    assert failed == 0



import requests

from insights.specs.default import DefaultSpecs
from insights.util.symbolic_name import get_spec_name_by_symbolic_name


def get_uploader_json():
    '''
    Download latest uploader.json to use for unit tests
    '''
    try:
        url = "https://cloud.redhat.com/api/v1/static/testing/uploader.v2.json"
        uploader_json = requests.get(url).json()
        return uploader_json
    except Exception:
        return


def test_all_symbolic_name_from_uploader_json():
    '''
    Verify that all symbolic names in uploader.json result as
    components in the output
    '''
    uploader_json = get_uploader_json()
    if not uploader_json:
        return

    # commands, files, globs
    for sl in uploader_json['commands'] + uploader_json['files'] + uploader_json['globs']:
        spec_name = get_spec_name_by_symbolic_name(sl['symbolic_name'])
        if spec_name:
            assert hasattr(DefaultSpecs, spec_name)



import doctest

from insights.parsers import ls_var_spool_clientmq
from insights.parsers.ls_var_spool_clientmq import LsVarSpoolClientmq
from insights.tests import context_wrap

LS_VAR_SPOOL_CLIENTMQ = """
total 40
-rw-rw---- 1 51 51   4 Jul 11 02:32 dfw6B6Wilr002718
-rw-rw---- 1 51 51   4 Jul 11 02:32 dfw6B6WixJ002715
-rw-rw---- 1 51 51   4 Jul 11 02:32 dfw6B6WjP6002721
-rw-rw---- 1 51 51 817 Jul 11 03:35 dfw6B7Z8BB002906
-rw-rw---- 1 51 51 817 Jul 11 04:02 dfw6B822T0011150
"""


def test_ls_var_spool_clientmq():
    ls_var_spool_clientmq = LsVarSpoolClientmq(context_wrap(LS_VAR_SPOOL_CLIENTMQ, path='insights_commands/ls_-ln_.var.spool.clientmqueue'))
    assert ls_var_spool_clientmq.files_of('/var/spool/clientmqueue') == ['dfw6B6Wilr002718', 'dfw6B6WixJ002715', 'dfw6B6WjP6002721', 'dfw6B7Z8BB002906', 'dfw6B822T0011150']
    onemail = ls_var_spool_clientmq.dir_entry('/var/spool/clientmqueue', 'dfw6B6Wilr002718')
    assert onemail is not None
    assert onemail == {'group': '51', 'name': 'dfw6B6Wilr002718', 'links': 1, 'perms': 'rw-rw----', 'raw_entry': '-rw-rw---- 1 51 51   4 Jul 11 02:32 dfw6B6Wilr002718', 'owner': '51', 'date': 'Jul 11 02:32', 'type': '-', 'dir': '/var/spool/clientmqueue', 'size': 4}


def test_ls_var_spool_clientmq_doc_examples():
    env = {
        'LsVarSpoolClientmq': LsVarSpoolClientmq,
        'ls_var_spool_clientmq': LsVarSpoolClientmq(context_wrap(LS_VAR_SPOOL_CLIENTMQ, path='insights_commands/ls_-ln_.var.spool.clientmqueue')),
    }
    failed, total = doctest.testmod(ls_var_spool_clientmq, globs=env)
    assert failed == 0



import pytest

from collections import defaultdict

from insights.core import filters
from insights.core.exceptions import SkipComponent
from insights.specs import Specs
from insights.specs.datasources.user_group import group_filters


def setup_function(func):
    if func is test_group_filters:
        filters.add_filter(Specs.group_info, ["wheel", "mem"])
    if func is test_group_filters_empty:
        filters.add_filter(Specs.group_info, [])


def teardown_function(func):
    filters._CACHE = {}
    filters.FILTERS = defaultdict(set)


def test_group_filters():
    broker = {}
    result = group_filters(broker)
    assert 'mem wheel' == result


def test_group_filters_empty():
    broker = {}
    with pytest.raises(SkipComponent):
        group_filters(broker)



"""
SatelliteMissedQueues - datasource ``satellite_missed_pulp_agent_queues``
=========================================================================
"""
from insights.core import Parser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.satellite_missed_pulp_agent_queues)
class SatelliteMissedQueues(Parser):
    """This parser parses the output of ``satellite_missed_pulp_agent_queues`` datasource.

    Typical output from the datasource is::

        pulp.agent.09008eec-aba6-4174-aa9f-e930004ce5c9:2018-01-16 00:06:13
        pulp.agent.fac7ebbc-ee4f-44b4-9fe0-3f4e42c7f024:2018-01-16 00:06:16
        0

    Examples:
        >>> satellite_queues.truncated
        False
        >>> 'pulp.agent.09008eec-aba6-4174-aa9f-e930004ce5c9' in satellite_queues.missed_queues
        True
        >>> satellite_queues.missed_queues['pulp.agent.09008eec-aba6-4174-aa9f-e930004ce5c9']
        '2018-01-16 00:06:13'

    Attributes:
        missed_queues(dict): Satellite missed pulp agent queues.
        truncated(bool): The missed queues truncated or not.

    Raises:
        SkipComponent: when no missed queues or the content isn't in expected format.
"""
    def parse_content(self, content):
        self.missed_queues = {}
        self.truncated = None
        if len(content) >= 2:
            for line in content[:-1]:
                if ':' not in line:
                    raise SkipComponent
                parts = line.split(':', 1)
                self.missed_queues[parts[0]] = parts[1]
            if content[-1].strip() in ['0', '1']:
                self.truncated = True if content[-1].strip() == '1' else False
        if not self.missed_queues or self.truncated is None:
            raise SkipComponent



from __future__ import print_function
import six
import sys

from datetime import datetime
from insights import dr, rule
from insights.util import utc


RENDERERS = {}
_FORMATTERS = {}


def get_formatter(name):
    """
    Looks up a formatter class given a prefix to it.
    The names are sorted, and the first matching class is returned.
    """
    for k in sorted(_FORMATTERS):
        if k.startswith(name):
            return _FORMATTERS[k]


class FormatterAdapterMeta(type):
    """ Automatically registers subclasses for later lookup. """

    def __init__(cls, name, bases, dct):
        if name not in ("FormatterAdapter", "EvaluatorFormatterAdapter"):
            _FORMATTERS[dr.get_name(cls)] = cls
        super(FormatterAdapterMeta, cls).__init__(name, bases, dct)


class FormatterAdapter(six.with_metaclass(FormatterAdapterMeta)):

    @staticmethod
    def configure(p):
        """ Override to add arguments to the ArgumentParser. """
        pass

    def __init__(self, args=None):
        """ Subclasses get passed the parsed args. """
        pass

    def preprocess(self, broker):
        """
        Called before any components have been run. Useful for registering
        observers.
        """
        pass

    def postprocess(self, broker):
        """
        Called after all components have been run. Useful for interrogating
        the broker for final state.
        """
        pass


class Formatter(object):
    def __init__(self, broker, stream=sys.stdout):
        self.broker = broker
        self.stream = stream
        self.start_time = datetime.now(utc)

    def __enter__(self):
        self.preprocess()
        return self

    def __exit__(self, _type, value, tb):
        self.postprocess()

    def preprocess(self):
        pass

    def postprocess(self):
        pass


class EvaluatorFormatterAdapter(FormatterAdapter):
    """
    Base class for formatters that want to serialize a SingleEvaluator after
    execution.
    """
    Impl = None

    @staticmethod
    def configure(p):
        p.add_argument("-m", "--missing", help="Show missing requirements.", action="store_true")
        p.add_argument("-r", "--render-content", help="Render rule content in json output.", action="store_true")
        p.add_argument("-S", "--show-rules", nargs="+",
                       choices=["fail", "info", "pass", "none", "metadata", "fingerprint"],
                       metavar="TYPE",
                       help="Show results per rule's type: 'fail', 'info', 'pass', 'none', 'metadata', and 'fingerprint'")
        p.add_argument("-F", "--fail-only",
                       help="Show FAIL results only. Conflict with '-m', will be dropped when using them together. This option is deprecated by '-S fail'",
                       action="store_true")

    def __init__(self, args=None):
        if args:
            hn = "insights.combiners.hostname, insights.parsers.client_metadata"
            args.plugins = ",".join([args.plugins, hn]) if args.plugins else hn
            self.missing = args.missing
            self.render_content = args.render_content
            fail_only = args.fail_only
            if args.missing and fail_only:
                # Drops the '-F' silently when specifying '-m' and '-F' together
                # --> Do NOT break the Format of the output
                fail_only = None
            self.show_rules = []  # Empty by default, means show ALL types (exclude "none")
            if not args.show_rules and fail_only:
                self.show_rules = ['rule']
            elif args.show_rules:
                self.show_rules = [opt.replace('fail', 'rule') for opt in args.show_rules]

    def preprocess(self, broker):
        self.formatter = self.Impl(broker, self.missing, self.render_content, self.show_rules)
        self.formatter.preprocess()

    def postprocess(self, broker):
        self.formatter.postprocess()


def get_content(obj, val):
    """
    Attempts to determine a jinja2 content template for a rule's response.
    """
    # does the rule define a content= kwarg?
    c = dr.get_delegate(obj).content

    # otherwise, does the rule module have a CONTENT attribute?
    if c is None:
        mod = sys.modules[obj.__module__]
        c = getattr(mod, "CONTENT", None)

    if c:
        # is the content a dictionary?
        if isinstance(c, dict):

            # does it contain a make_* class as a key?
            v = c.get(val.__class__)
            if v is not None:
                return v

            # does it contain an error key?
            key = val.get_key()
            if key:
                v = c.get(key)

                # is the value a dict that contains make_* classes?
                if isinstance(v, dict):
                    return v.get(val.__class__)
                return v
        else:
            return c


try:
    from jinja2 import Template
    from jinja2 import exceptions as jinja2Exceptions

    def format_rule(comp, val):
        content = get_content(comp, val)
        if content and val.get("type") != "skip":
            try:
                text = Template(content).render(val)
            except jinja2Exceptions.UndefinedError as err:
                text = '\n'.join([str(val),
                                "CONTENT:",
                                "--------",
                                content,
                                "Failed to render the Content above:",
                                "    " + str(err)])
            finally:
                return text
        return str(val)

    RENDERERS[rule] = format_rule
except:
    pass


def render(comp, val):
    _type = dr.get_component_type(comp)
    func = RENDERERS.get(_type)
    return func(comp, val) if func else str(val)


def get_response_of_types(response, missing=True, show_rules=None):
    # Check the "-m" option:
    #  - When "-m" is specified, show the "skips" rules
    #  - When "-m" is NOT specified, do not show the "skips" rules
    if not missing and 'skips' in response:
        response.pop('skips')
    # Check the "-S" option:
    #  - When "-m" is specified but "-S" is NOT specified, show all the loaded rules
    #  - When neither "-m" nor "-S" is specified, show all the HIT rules (exclude the "skips")
    if not show_rules:
        #  - Discard the "make_none" by default when no "-S"
        #  That means show "make_none" rules only when "none" is specified in "-S"
        response.pop('none') if 'none' in response else None
        return response
    #  - Discard the "medadata" rules when it's not specified in the "-S" option
    if 'metadata' not in show_rules and 'metadata' in response.get('system', {}):
        response['system'].pop('metadata')
    #  - Discard the "make_fail" rules when it's not specified in the "-S" option
    if 'rule' not in show_rules and 'reports' in response:
        response.pop('reports')
    #  - Discard the "make_info" rules when it's not specified in the "-S" option
    if 'info' not in show_rules and 'info' in response:
        response.pop('info')
    #  - Discard the "make_pass" rules when it's not specified in the "-S" option
    if 'pass' not in show_rules and 'pass' in response:
        response.pop('pass')
    #  - Discard the "make_none" rules when it's not specified in the "-S" option
    if 'none' not in show_rules and 'none' in response:
        response.pop('none')
    #  - Discard the "fingerprint" rules when it's not specified in the "-S" option
    if 'fingerprint' not in show_rules and 'fingerprints' in response:
        response.pop('fingerprints')
    return response



from insights.parsers.sysconfig import KernelSysconfig
from insights.tests import context_wrap

SYSCONFIG_KERNEL = """
# UPDATEDEFAULT specifies if new-kernel-pkg should make
# new kernels the default
UPDATEDEFAULT=yes

# DEFAULTKERNEL specifies the default kernel package type
DEFAULTKERNEL=kernel

# MAKEDEBUG specifies if new-kernel-pkg should create non-default
# "debug" entries for new kernels.
MAKEDEBUG=yes
""".strip()


def test_sysconfig_nfs():
    result = KernelSysconfig(context_wrap(SYSCONFIG_KERNEL))
    assert result['UPDATEDEFAULT'] == 'yes'
    assert result.get('DEFAULTKERNEL') == 'kernel'
    assert result.get('MAKEDEBUG') == 'yes'



from insights.parsers.fapolicyd_rules import FapolicydRules
from insights.tests import context_wrap

CONTENT = """
# This file contains the list of all patterns. Only the ld_so pattern
# is enabled by default.

deny_audit perm=any pattern=ld_so : all
#deny_audit perm=any pattern=ld_preload : all
#deny_audit perm=any pattern=static : all
"""


def test_udev_rules():
    FapolicydRules.last_scan('test_deny_audit_lo_so1', 'deny_audit perm=any pattern=ld_so : all')
    result = FapolicydRules(context_wrap(CONTENT, path='/etc/fapolicyd/rules.d/30-patterns.rules'))
    assert result.test_deny_audit_lo_so1.get('raw_line') == 'deny_audit perm=any pattern=ld_so : all'


def test_doc():
    """
    To make the examples readable, it's better to show one of the main usage "last_scan".
    And the last_scan should be called before the parser initialation.
    However, the initialization is done here, so the "last_scan" in the example won't work.
    As a result, it will rasise error when refering the result key.
    So we will not thest the examples, just show the users how to use it.
    """
    pass



"""
``/sys/bus`` Device Usage Information
=====================================

A parser to parse the usage information of devices connected
on sys/bus.

Parsers included in this module are:

CdcWDM - file ``/sys/bus/usb/drivers/cdc_wdm/module/refcnt``
------------------------------------------------------------

"""
from insights.core import Parser
from insights.core.exceptions import ParseException
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.cdc_wdm)
class CdcWDM(Parser):

    """
    This file `/sys/bus/usb/drivers/cdc_wdm/module/refcnt` contains
    device usage count, i.e if a device is in use then the non-zero
    value will be present in the file.

    Sample Content::

        0 - Not in use.
        1 - Device is opened and it is in use.

    Examples::

        >>> type(device_usage)
        <class 'insights.parsers.sys_bus.CdcWDM'>
        >>> device_usage.device_usage_cnt
        1
        >>> device_usage.device_in_use
        True

    Raises:
        SkipComponent: When contents are empty
        ParseException: When contents are invalid
    """

    def parse_content(self, content):
        try:
            self._ref_count = int(content[0].strip())
        except Exception:
            raise ParseException("Invalid Content!")

    @property
    def device_usage_cnt(self):
        """
        Returns (int): device usage count.
        """
        return self._ref_count

    @property
    def device_in_use(self):
        """
        Returns (bool): ``True`` when device in use else ``False``.
        """
        return True if self._ref_count else False



from insights.parsers.ovs_vsctl_show import OVSvsctlshow
from insights.tests import context_wrap

ovs_vsctl_show_output = """
e4d4f521-086d-4479-a88f-d531cd1646b8
    Bridge br-ex
        Port br-ex
            Interface br-ex
                type: internal
        Port "qg-dacc6089-be"
            Interface "qg-dacc6089-be"
                type: internal
        Port "eth0"
            Interface "eth0"
        Port phy-br-ex
            Interface phy-br-ex
                type: patch
                options: {peer=int-br-ex}
    Bridge br-int
        fail_mode: secure
        Port "tapfcce898c-ca"
            tag: 1
            Interface "tapfcce898c-ca"
                type: internal
        Port int-br-ex
            Interface int-br-ex
                type: patch
                options: {peer=phy-br-ex}
        Port "tapdf2d6113-b2"
            tag: 2
            Interface "tapdf2d6113-b2"
                type: internal
        Port patch-tun
            Interface patch-tun
                type: patch
                options: {peer=patch-int}
        Port br-int
            Interface br-int
                type: internal
        Port "ha-1f423581-cd"
            tag: 3
            Interface "ha-1f423581-cd"
                type: internal
        Port "qr-417e232b-dd"
            tag: 1
            Interface "qr-417e232b-dd"
                type: internal
    Bridge br-tun
        fail_mode: secure
        Port "vxlan-aca80118"
            Interface "vxlan-aca80118"
                type: vxlan
                options: {df_default="true", in_key=flow, local_ip="172.168.1.26", out_key=flow, remote_ip="172.168.1.24"}
        Port "vxlan-aca80119"
            Interface "vxlan-aca80119"
                type: vxlan
                options: {df_default="true", in_key=flow, local_ip="172.168.1.26", out_key=flow, remote_ip="172.168.1.25"}
        Port "vxlan-aca80117"
            Interface "vxlan-aca80117"
                type: vxlan
                options: {df_default="true", in_key=flow, local_ip="172.168.1.26", out_key=flow, remote_ip="172.168.1.23"}
        Port patch-int
            Interface patch-int
                type: patch
                options: {peer=patch-tun}
        Port "vxlan-aca80116"
            Interface "vxlan-aca80116"
                type: vxlan
                options: {df_default="true", in_key=flow, local_ip="172.168.1.26", out_key=flow, remote_ip="172.168.1.22"}
        Port br-tun
            Interface br-tun
                type: internal
    ovs_version: "2.3.2"
""".strip()

ovs_vsctl_show_missing_lines = """
e4d4f521-086d-4479-a88f-d531cd1646b8
    Bridge br-ex
"""


def test_ovs_vsctl_show():
    ovs_ctl_cls = OVSvsctlshow(context_wrap(ovs_vsctl_show_output))
    assert ovs_ctl_cls.get_ovs_version() == "2.3.2"
    assert ovs_ctl_cls.get_bridge("br-int").get("fail_mode") == "secure"
    br_tun = ovs_ctl_cls.get_bridge("br-tun")
    assert br_tun.get("fail_mode") == "secure"
    assert len(br_tun.get("ports")) == 6
    ports = br_tun.get("ports")
    assert ports[0].get("interface") == "vxlan-aca80118"
    assert ports[0].get("type") == "vxlan"
    options = ports[0].get("options")
    assert options.get("df_default") == "true"
    assert options.get("local_ip") == "172.168.1.26"
    assert options.get("out_key") == "flow"

    bad = OVSvsctlshow(context_wrap(ovs_vsctl_show_missing_lines))
    assert not hasattr(bad, 'data')



import doctest

from insights.parsers import yum_conf
from insights.tests import context_wrap


YUM_CONF_DOC_TEST = """
[main]
cachedir=/var/cache/yum/$basearch/$releasever
keepcache=0
debuglevel=2
logfile=/var/log/yum.log
exactarch=1
obsoletes=1
gpgcheck=1
plugins=1
installonly_limit=3

[rhel-7-server-rpms]
metadata_expire = 86400
baseurl = https://cdn.redhat.com/content/rhel/server/7/$basearch
name = Red Hat Enterprise Linux 7 Server (RPMs)
gpgkey = file:///etc/pki/rpm-gpg/RPM-GPG-KEY-redhat-release
enabled = 1
gpgcheck = 1
""".strip()

YUM_CONF = """
[main]
cachedir=/var/cache/yum/$basearch/$releasever
keepcache=0
debuglevel=2
logfile=/var/log/yum.log
exactarch=1
obsoletes=1
gpgcheck=1
plugins=1
installonly_limit=3

#  This is the default, if you make this bigger yum won't see if the metadata
# is newer on the remote and so you'll "gain" the bandwidth of not having to
# download the new metadata and "pay" for it by yum not having correct
# information.
#  It is esp. important, to have correct metadata, for distributions like
# Fedora which don't keep old packages around. If you don't like this checking
# interupting your command line usage, it's much better to have something
# manually check the metadata once an hour (yum-updatesd will do this).
# metadata_expire=90m

# PUT YOUR REPOS HERE OR IN separate files named file.repo
# in /etc/yum.repos.d

[rhel-7-server-rhn-tools-beta-debug-rpms]
metadata_expire = 86400
sslclientcert = /etc/pki/entitlement/1234.pem
baseurl = https://cdn.redhat.com/content/beta/rhel/server/7/$basearch/rhn-tools/debug
ui_repoid_vars = basearch
sslverify = 1
name = RHN Tools for Red Hat Enterprise Linux 7 Server Beta (Debug RPMs)
sslclientkey = /etc/pki/entitlement/1234-key.pem
gpgkey = file:///etc/pki/rpm-gpg/RPM-GPG-KEY-redhat-beta,file:///etc/pki/rpm-gpg/RPM-GPG-KEY-redhat-release
enabled = 0
sslcacert = /etc/rhsm/ca/redhat-uep.pem
gpgcheck = 1

[bad-repo]
gpgkey =
"""


CONF_PATH = '/etc/yum.conf'


def test_doc_examples():
    env = {
        'yconf': yum_conf.YumConf(context_wrap(YUM_CONF_DOC_TEST))
    }
    failed, total = doctest.testmod(yum_conf, globs=env)
    assert failed == 0


def test_get_yum_conf():
    results = yum_conf.YumConf(context_wrap(YUM_CONF, path=CONF_PATH))

    assert results.items('main') == {
        'plugins': '1',
        'keepcache': '0',
        'cachedir': '/var/cache/yum/$basearch/$releasever',
        'exactarch': '1',
        'obsoletes': '1',
        'installonly_limit': '3',
        'debuglevel': '2',
        'gpgcheck': '1',
        'logfile': '/var/log/yum.log'
    }

    assert results.items('rhel-7-server-rhn-tools-beta-debug-rpms') == {
        u'ui_repoid_vars': u'basearch',
        u'sslverify': u'1',
        u'name': u'RHN Tools for Red Hat Enterprise Linux 7 Server Beta (Debug RPMs)',
        u'sslclientkey': u'/etc/pki/entitlement/1234-key.pem',
        u'enabled': u'0',
        u'gpgkey': [u'file:///etc/pki/rpm-gpg/RPM-GPG-KEY-redhat-beta',
                    u'file:///etc/pki/rpm-gpg/RPM-GPG-KEY-redhat-release'],
        u'sslclientcert': u'/etc/pki/entitlement/1234.pem',
        u'baseurl': [u'https://cdn.redhat.com/content/beta/rhel/server/7/$basearch/rhn-tools/debug'],
        u'sslcacert': u'/etc/rhsm/ca/redhat-uep.pem',
        u'gpgcheck': u'1',
        u'metadata_expire': u'86400'
    }

    assert results.file_name == 'yum.conf'
    assert results.file_path == '/etc/yum.conf'



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import findmnt
from insights.tests import context_wrap

FINDMNT_NETNS_SHARED_1 = """
TARGET                                                          SOURCE                                FSTYPE          OPTIONS                                                                       PROPAGATION
/sys                                                            sysfs                                 sysfs           rw,nosuid,nodev,noexec,relatime,seclabel                                      shared
/proc                                                           proc                                  proc            rw,nosuid,nodev,noexec,relatime                                               shared
/dev                                                            devtmpfs                              devtmpfs        rw,nosuid,seclabel,size=8035516k,nr_inodes=2008879,mode=755                   shared
/sys/kernel/security                                            securityfs                            securityfs      rw,nosuid,nodev,noexec,relatime                                               shared
/dev/shm                                                        tmpfs                                 tmpfs           rw,nosuid,nodev,seclabel                                                      shared
/run/netns                                                      tmpfs[/netns]                         tmpfs           rw,nosuid,nodev,seclabel,mode=755                                             shared
/netns                                                          tmpfs[/netns]                         tmpfs           rw,nosuid,nodev,seclabel,mode=755                                             shared
/run/netns/qdhcp-08f32dab-927e-4a61-933d-57d425827b57           proc                                  proc            rw,nosuid,nodev,noexec,relatime                                               shared
/run/netns/qdhcp-fd138c0a-5ec7-44f8-88df-0501c4c7a968           proc                                  proc            rw,nosuid,nodev,noexec,relatime                                               shared
""".strip()

FINDMNT_NETNS_SHARED_2 = """
TARGET                                                          SOURCE                                FSTYPE          OPTIONS                                                                       PROPAGATION
/sys                                                            sysfs                                 sysfs           rw,nosuid,nodev,noexec,relatime,seclabel                                      shared
/proc                                                           proc                                  proc            rw,nosuid,nodev,noexec,relatime                                               shared
/dev                                                            devtmpfs                              devtmpfs        rw,nosuid,seclabel,size=8035516k,nr_inodes=2008879,mode=755                   shared
/sys/kernel/security                                            securityfs                            securityfs      rw,nosuid,nodev,noexec,relatime                                               shared
/dev/shm                                                        tmpfs                                 tmpfs           rw,nosuid,nodev,seclabel                                                      shared
/netns                                                          tmpfs[/netns]                         tmpfs           rw,nosuid,nodev,seclabel,mode=755                                             shared
/run/netns/qdhcp-08f32dab-927e-4a61-933d-57d425827b57           proc                                  proc            rw,nosuid,nodev,noexec,relatime                                               shared
/run/netns/qdhcp-fd138c0a-5ec7-44f8-88df-0501c4c7a968           proc                                  proc            rw,nosuid,nodev,noexec,relatime                                               shared
""".strip()

FINDMNT_NETNS_PRIVATE = """
TARGET                                                          SOURCE                                FSTYPE          OPTIONS                                                                       PROPAGATION
/sys                                                            sysfs                                 sysfs           rw,nosuid,nodev,noexec,relatime,seclabel                                      shared
/proc                                                           proc                                  proc            rw,nosuid,nodev,noexec,relatime                                               shared
/dev                                                            devtmpfs                              devtmpfs        rw,nosuid,seclabel,size=8035516k,nr_inodes=2008879,mode=755                   shared
/sys/kernel/security                                            securityfs                            securityfs      rw,nosuid,nodev,noexec,relatime                                               shared
/dev/shm                                                        tmpfs                                 tmpfs           rw,nosuid,nodev,seclabel                                                      shared
/run/netns                                                      tmpfs[/netns]                         tmpfs           rw,nosuid,nodev,seclabel,mode=755                                             private
/netns                                                          tmpfs[/netns]                         tmpfs           rw,nosuid,nodev,seclabel,mode=755                                             private
/run/netns/qdhcp-08f32dab-927e-4a61-933d-57d425827b57           proc                                  proc            rw,nosuid,nodev,noexec,relatime                                               private
/run/netns/qdhcp-fd138c0a-5ec7-44f8-88df-0501c4c7a968           proc                                  proc            rw,nosuid,nodev,noexec,relatime                                               private
""".strip()

FINDMNT_NO_NETNS = """
TARGET                                                                                           SOURCE     FSTYPE     OPTIONS                                                                                                                                PROPAGATION
/sys                                                                                             sysfs      sysfs      rw,nosuid,nodev,noexec,relatime,seclabel                                                                                               shared
/proc                                                                                            proc       proc       rw,nosuid,nodev,noexec,relatime                                                                                                        shared
/dev                                                                                             devtmpfs   devtmpfs   rw,nosuid,seclabel,size=5912356k,nr_inodes=1478089,mode=755                                                                            shared
/sys/kernel/security                                                                             securityfs securityfs rw,nosuid,nodev,noexec,relatime                                                                                                        shared
/dev/shm                                                                                         tmpfs      tmpfs      rw,nosuid,nodev,seclabel                                                                                                               shared
/dev/pts                                                                                         devpts     devpts     rw,nosuid,noexec,relatime,seclabel,gid=5,mode=620,ptmxmode=000                                                                         shared
/run                                                                                             tmpfs      tmpfs      rw,nosuid,nodev,seclabel,mode=755                                                                                                      shared
/sys/fs/cgroup                                                                                   tmpfs      tmpfs      ro,nosuid,nodev,noexec,seclabel,mode=755                                                                                               shared
/sys/fs/cgroup/systemd                                                                           cgroup     cgroup     rw,nosuid,nodev,noexec,relatime,seclabel,xattr,release_agent=/usr/lib/systemd/systemd-cgroups-agent,name=systemd                       shared
/sys/fs/pstore                                                                                   pstore     pstore     rw,nosuid,nodev,noexec,relatime                                                                                                        shared
/sys/fs/cgroup/perf_event                                                                        cgroup     cgroup     rw,nosuid,nodev,noexec,relatime,seclabel,perf_event                                                                                    shared
/sys/fs/cgroup/cpu,cpuacct                                                                       cgroup     cgroup     rw,nosuid,nodev,noexec,relatime,seclabel,cpuacct,cpu                                                                                   shared
/sys/fs/cgroup/hugetlb                                                                           cgroup     cgroup     rw,nosuid,nodev,noexec,relatime,seclabel,hugetlb                                                                                       shared
/sys/fs/cgroup/freezer                                                                           cgroup     cgroup     rw,nosuid,nodev,noexec,relatime,seclabel,freezer                                                                                       shared
/sys/fs/cgroup/net_cls,net_prio                                                                  cgroup     cgroup     rw,nosuid,nodev,noexec,relatime,seclabel,net_prio,net_cls                                                                              shared
/sys/fs/cgroup/devices                                                                           cgroup     cgroup     rw,nosuid,nodev,noexec,relatime,seclabel,devices                                                                                       shared
/sys/fs/cgroup/blkio                                                                             cgroup     cgroup     rw,nosuid,nodev,noexec,relatime,seclabel,blkio                                                                                         shared
/var/lib/docker/containers/eb7baaab48d9757e6b6250303c078862792d2a8d4cebd7438170057044a90c3e/shm  shm        tmpfs      rw,nosuid,nodev,noexec,relatime,seclabel,size=65536k                                                                                   private
/var/lib/docker/containers/8af84c3469728812b2959c4a1c95336159efd17407a51c4461b5e20f09a5bb6f/shm  shm        tmpfs      rw,nosuid,nodev,noexec,relatime,seclabel,size=65536k                                                                                   private
/var/lib/docker/containers/54f2fdc2885805159d9fc2c656a3fa073b99bcfa430d9b42683afcc5f5b7dc4d/shm  shm        tmpfs      rw,nosuid,nodev,noexec,relatime,seclabel,size=65536k                                                                                   private
/var/lib/docker/containers/752bbb082c2b7f3d411aa244b77e2d57f64ca9ec25cb954a3e4e97d48baaa542/shm  shm        tmpfs      rw,nosuid,nodev,noexec,relatime,seclabel,size=65536k                                                                                   private
/var/lib/docker/containers/867c62f7157e8c7fff23b81a19c2e4368a2bf5703383ab2372fabb9cb38e8a4a/shm  shm        tmpfs      rw,nosuid,nodev,noexec,relatime,seclabel,size=65536k                                                                                   private
/var/lib/docker/containers/3e1814279bf5f42a74288d8e3a197f7b1388f59582590045755e8e4ca8ee5a19/shm  shm        tmpfs      rw,nosuid,nodev,noexec,relatime,seclabel,size=65536k                                                                                   private
/var/lib/docker/containers/64136ecd23d34617ddb5e18eb337b1467a8f5fa29e804420f4c61688ae2e6531/shm  shm        tmpfs      rw,nosuid,nodev,noexec,relatime,seclabel,size=65536k                                                                                   private
/var/lib/docker/containers/fe2b38088424c5102fdab3f13f93c9341c9311dca176eb175a58bb8bdf545927/shm  shm        tmpfs      rw,nosuid,nodev,noexec,relatime,seclabel,size=65536k                                                                                   private
""".strip()


def test_findmnt_output():
    output = findmnt.FindmntPropagation(context_wrap(FINDMNT_NETNS_SHARED_1))
    assert output.search_target('shm') == [{'target': '/dev/shm', 'source': 'tmpfs', 'fstype': 'tmpfs', 'options': 'rw,nosuid,nodev,seclabel', 'propagation': 'shared'}]
    assert len(output.target_startswith('/run/netns')) == 3
    assert output.target_startswith('/run/netns')[0].get('propagation') == 'shared'

    output = findmnt.FindmntPropagation(context_wrap(FINDMNT_NETNS_SHARED_2))
    assert output.target_startswith('/run/netns')[0].get('propagation') == 'shared'

    output = findmnt.FindmntPropagation(context_wrap(FINDMNT_NETNS_PRIVATE))
    assert output.target_startswith('/run/netns')[0].get('propagation') != 'shared'
    assert output.target_startswith('/run/netns')[0].get('propagation') == 'private'

    output = findmnt.FindmntPropagation(context_wrap(FINDMNT_NO_NETNS))
    assert output.search_target('netns') == []
    assert output.target_startswith('/run/netns') == []


def test_blank_output():
    with pytest.raises(SkipComponent) as e:
        findmnt.FindmntPropagation(context_wrap(""))
    assert "No data." in str(e)


def test_documentation():
    failed_count, tests = doctest.testmod(
        findmnt,
        globs={'output': findmnt.FindmntPropagation(context_wrap(FINDMNT_NETNS_SHARED_1))}
    )
    assert failed_count == 0



"""
User and Group
==============

Module for processing the ``user`` and ``group`` of Linux.

Parsers provided by this module include:

GroupInfo - command ``getent group <groupname>``
------------------------------------------------

"""
from insights.core import CommandParser
from insights.core.exceptions import ParseException, SkipComponent
from insights.core.plugins import parser
from insights.parsers import keyword_search
from insights.specs import Specs


@parser(Specs.group_info)
class GroupInfo(CommandParser, list):
    """
    Class to parse the output of ``getent group <groupname>``. The list of
    `groupname` is getting from the spec filters specified in rules.

    Typical output of the ``getent group <groupname>`` command is::

        wheel:x:10:admin,tester
        mem:x:8:

    Examples:

    >>> type(grp)
    <class 'insights.parsers.user_group.GroupInfo'>
    >>> grp[0]['id']
    10
    >>> grp[0]['name']
    'wheel'
    >>> grp[0]['users']
    ['admin', 'tester']
    """
    def parse_content(self, content):
        if not content:
            raise SkipComponent

        for line in content:
            try:
                row = line.split(':')
                self.append(
                    dict(
                        name=row[0],
                        id=int(row[2]),
                        users=row[-1].split(',') if row[-1] else []
                    )
                )
            except Exception as ex:
                raise ParseException(ex)

    def search(self, **kwargs):
        """
        Get the sub-list containing the keywords by searching the output of
        ``getent group <groupname>``

        This uses the :py:func:`insights.parsers.keyword_search` function for searching,
        see its documentation for usage details. If no search parameters are given or does
        match the search, then nothing will be returned.

        Returns:
            list: A list of dictionaries of the ``getent group <groupname>``
                  content that match the given search criteria.

        Examples:

        >>> grp.search(name='mem')[0] == grp[1]
        True
        >>> grp.search(users__contains='admin')[0] == grp[0]
        True
        """
        return keyword_search(self, **kwargs)



# -*- coding: utf-8 -*-

import doctest

from collections import defaultdict

from insights.core import filters
from insights.parsers import dse_ldif_simple
from insights.parsers.dse_ldif_simple import DseLdifSimple
from insights.specs import Specs
from insights.tests import context_wrap

DSE_LDIF_REAL_EXAMPLE = """

dn: cn=config
cn: config
objectClass: top
objectClass: extensibleObject
objectClass: nsslapdConfig
nsslapd-schemadir: /etc/dirsrv/slapd-dir/schema
nsslapd-lockdir: /var/lock/dirsrv/slapd-dir
nsslapd-tmpdir: /tmp
nsslapd-certdir: /etc/dirsrv/slapd-dir
nsslapd-ldifdir: /var/lib/dirsrv/slapd-dir/ldif
nsslapd-bakdir: /var/lib/dirsrv/slapd-dir/bak
nsslapd-rundir: /var/run/dirsrv
nsslapd-instancedir: /usr/lib64/dirsrv/slapd-dir
nsslapd-accesslog: /var/log/dirsrv/slapd-dir/access
nsslapd-localhost: testinstance.local
nsslapd-port: 389
nsslapd-localuser: dirsrv
nsslapd-errorlog: /var/log/dirsrv/slapd-dir/errors
nsslapd-auditlog: /var/log/dirsrv/slapd-dir/audit
nsslapd-auditfaillog: /var/log/dirsrv/slapd-dir/audit
nsslapd-rootdn: cn=Directory Manager
nsslapd-ldapifilepath: /var/run/slapd-dir.socket
nsslapd-ldapilisten: off
nsslapd-ldapiautobind: off
nsslapd-ldapimaprootdn: cn=Directory Manager
nsslapd-ldapimaptoentries: off
nsslapd-ldapiuidnumbertype: uidNumber
nsslapd-ldapigidnumbertype: gidNumber
nsslapd-ldapientrysearchbase: dc=testinstance,dc=local
nsslapd-defaultnamingcontext: dc=testinstance,dc=local
aci: (targetattr="*")(version 3.0; acl "Configuration Administrators Group"; a
 llow (all) groupdn="ldap:///cn=Configuration Administrators,ou=Groups,ou=Topo
 logyManagement,o=NetscapeRoot";)
aci: (targetattr="*")(version 3.0; acl "Configuration Administrator"; allow (a
 ll) userdn="ldap:///uid=admin,ou=Administrators,ou=TopologyManagement,o=Netsc
 apeRoot";)
aci: (targetattr = "*")(version 3.0; acl "SIE Group"; allow (all) groupdn = "l
 dap:///cn=slapd-dir,cn=Red Hat Directory Server,cn=Server Group,cn=testinstan
 ce.local,ou=testinstance.local,o=NetscapeRoot";)
modifiersName: cn=directory manager
modifyTimestamp: 20211015231914Z
nsslapd-securePort: 636
nsslapd-security: on
nsslapd-rootpw: {SSHA512}la6KbVGTR3XKHN6CoZ/PcYvOrS7qGAVSC9kjvGtyuPzSJGbHXReTs
 FUBF6QnP0jHAONEx4784x6PNPcMzTOdpoJw0gOQkXKM
numSubordinates: 10


dn: cn=monitor
objectClass: top
objectClass: extensibleObject
cn: monitor
aci: (target ="ldap:///cn=monitor*")(targetattr != "aci || connection")(versio
 n 3.0; acl "monitor"; allow( read, search, compare ) userdn = "ldap:///anyone
 ";)
creatorsName: cn=server,cn=plugins,cn=config
modifiersName: cn=server,cn=plugins,cn=config
createTimestamp: 20211015215626Z
modifyTimestamp: 20211015215626Z
numSubordinates: 3

dn: cn=encryption,cn=config
objectClass: top
objectClass: nsEncryptionConfig
cn: encryption
nsSSLSessionTimeout: 0
nsSSLClientAuth: allowed
sslVersionMin: TLS1.0
sslVersionMax: TLS1.1
nsSSL3: on
nsTLS1: on
allowWeakCipher: on
nsSSL3Ciphers: +all
creatorsName: cn=server,cn=plugins,cn=config
modifiersName: cn=server,cn=plugins,cn=config
createTimestamp: 20211015215626Z
modifyTimestamp: 20211015231142Z
CACertExtractFile: /etc/dirsrv/slapd-dir/server-cert.pem
numSubordinates: 1

dn: cn=features,cn=config
objectClass: top
objectClass: nsContainer
cn: features
numSubordinates: 5

"""

DSE_LDIF_SMOKE = """
cn: config
nsslapd-security: on
sslVersionMin: TLS1.0
sslVersionMax: TLS1.1
nsSSL3: on
# a comment : with a colon
"""

# This is purely for coverage testing. Real deployments don't use these features
# for these attributes.
DSE_LDIF_COVERAGE = """
# a comment
nsslapd-security: o
 n
sslVersionMax:: VExTMS4yIA==
sslVersionMin:< file:///tmp/somefile
nsSSL3: on
"""

DSE_LDIF_DOCTEST = """
dn: cn=config
nsslapd-securePort: 636
nsslapd-security: on

dn: cn=encryption,cn=config
sslVersionMin: SSLv3
sslVersionMax: TLS1.1
nsSSL3: on
"""


def setup_function(func):
    if func in [test_dse_ldif_filtered]:
        filters.add_filter(
            Specs.dse_ldif, [
                "nsslapd-security",
                "sslVersionMin",
                "sslVersionMax",
                "nsSSL3",
                "cn: config",  # Note that this can serve as a canary for knowing whether the spec is collected.
            ]
        )


def teardown_function(func):
    filters._CACHE = {}
    filters.FILTERS = defaultdict(set)


def test_dse_ldif_smoke():
    dse_ldif_simple = DseLdifSimple(context_wrap(DSE_LDIF_SMOKE))
    assert None is dse_ldif_simple.get("asdf")
    assert len(dse_ldif_simple) == 5
    expected = {
        "cn": "config",  # just a canary to detect spec collection status
        "nsslapd-security": "on",
        "sslVersionMin": "TLS1.0",
        "sslVersionMax": "TLS1.1",
        "nsSSL3": "on",
    }
    assert dict(dse_ldif_simple) == expected
    assert dse_ldif_simple["nsslapd-security"] == "on"
    assert "sslVersionMin" in dse_ldif_simple
    for k in dse_ldif_simple:
        assert expected[k] == dse_ldif_simple[k]
        assert expected[k] == dse_ldif_simple.get(k)


def test_dse_ldif_coverage():
    dse_ldif_simple = DseLdifSimple(context_wrap(DSE_LDIF_COVERAGE))
    assert "sslVersionMin" not in dse_ldif_simple
    assert "sslVersionMax" not in dse_ldif_simple
    assert "nsSSL3" in dse_ldif_simple
    assert len(dse_ldif_simple) == 2
    expected = {
        "nsSSL3": "on",
        # This doesn't happen in real deployments because 389-ds automatically
        # reformats it back to a single line.
        "nsslapd-security": "o",
    }
    assert dict(dse_ldif_simple) == expected


def test_dse_ldif_filtered():
    dse_ldif_simple = DseLdifSimple(context_wrap(DSE_LDIF_REAL_EXAMPLE, filtered_spec=Specs.dse_ldif))
    assert dse_ldif_simple["nsslapd-security"] == "on"
    assert len(dse_ldif_simple) == 6
    expected = {
        "cn": "config",  # just a canary to detect spec collection status
        "nsslapd-security": "on",
        "sslVersionMin": "TLS1.0",
        "sslVersionMax": "TLS1.1",
        "nsSSL3": "on",
        "nsSSL3Ciphers": "+all",
    }
    assert dict(dse_ldif_simple) == expected


def test_doc_examples():
    env = {
        "dse_ldif_simple": DseLdifSimple(context_wrap(DSE_LDIF_DOCTEST)),
    }
    failed, total = doctest.testmod(dse_ldif_simple, globs=env)
    assert failed == 0



import csv
import json
import os

from mock.mock import patch
from pytest import mark

from insights.client.archive import InsightsArchive
from insights.client.config import InsightsConfig
from insights.core.spec_cleaner import Cleaner

hostname = "report.test.com"
test_file_data = 'ip: 10.0.2.155\ntestword\n{0}'.format(hostname)


@mark.parametrize(
    ("obfuscate", "obfuscate_hostname"),
    [
        (False, False), (True, False), (True, True),
    ]
)
@mark.parametrize("test_umask", [0o000, 0o022])
def test_rhsm_facts(test_umask, obfuscate, obfuscate_hostname):
    rhsm_facts_file = '/tmp/insights_test_rhsm.facts'
    conf = InsightsConfig(obfuscate=obfuscate,
                          obfuscate_hostname=obfuscate_hostname)
    conf.rhsm_facts_file = rhsm_facts_file
    arch = InsightsArchive(conf)
    arch.create_archive_dir()

    # put something in the archive to redact
    test_file = os.path.join(arch.archive_dir, 'test.file')
    with open(test_file, 'w') as t:
        t.write(test_file_data)

    old_umask = os.umask(test_umask)
    pp = Cleaner(conf, {'keywords': ['testword']}, hostname)
    pp.clean_file(test_file, [])
    pp.generate_report(arch.archive_name)
    arch.delete_archive_dir()

    umask_after_test = os.umask(old_umask)

    assert os.path.isfile(rhsm_facts_file)
    st = os.stat(rhsm_facts_file)
    assert st.st_mode & 0o777 == 0o644 & ~test_umask
    assert umask_after_test == test_umask  # umask was not changed by Cleaner
    with open(rhsm_facts_file, 'r') as fp:
        facts = json.load(fp)
        # hostname
        assert facts['insights_client.hostname'] == hostname
        assert facts['insights_client.obfuscate_hostname_enabled'] == obfuscate_hostname
        hns = json.loads(facts['insights_client.hostnames'])
        if obfuscate_hostname:
            assert hns[0]['original'] == hostname
            assert '.example.com' in hns[0]['obfuscated']
        else:
            hns == []
        # ip
        assert facts['insights_client.obfuscate_ip_enabled'] == obfuscate
        ips = json.loads(facts['insights_client.ips'])
        if obfuscate:
            assert ips[0]['original'] == '10.0.2.155'
            assert ips[0]['obfuscated'] == '10.230.230.1'
        else:
            assert ips == []
        # keyword
        kws = json.loads(facts['insights_client.keywords'])
        assert kws[0]['original'] == 'testword'
        assert kws[0]['obfuscated'] == 'keyword0'

    os.unlink(rhsm_facts_file)


@mark.parametrize("rm_conf", [{}, {'keywords': ['testword']}])
@mark.parametrize(
    ("obfuscate", "obfuscate_hostname"),
    [
        (False, False), (True, False), (True, True),
    ]
)
@patch('insights.core.spec_cleaner.Cleaner.generate_rhsm_facts', return_value=None)
def test_all_csv_reports(rhsm_facts, obfuscate, rm_conf, obfuscate_hostname):
    conf = InsightsConfig(obfuscate=obfuscate,
                          obfuscate_hostname=obfuscate_hostname)
    arch = InsightsArchive(conf)
    arch.create_archive_dir()

    # put something in the archive to redact
    test_file = os.path.join(arch.archive_dir, 'test.file')
    with open(test_file, 'w') as t:
        t.write(test_file_data)

    pp = Cleaner(conf, rm_conf, hostname)
    ip_report_file = os.path.join(pp.report_dir, "%s-ip.csv" % arch.archive_name)
    hn_report_file = os.path.join(pp.report_dir, "%s-hostname.csv" % arch.archive_name)
    kw_report_file = os.path.join(pp.report_dir, "%s-keyword.csv" % arch.archive_name)
    if os.path.isfile(ip_report_file):
        os.unlink(ip_report_file)
    if os.path.isfile(hn_report_file):
        os.unlink(hn_report_file)
    if os.path.isfile(kw_report_file):
        os.unlink(kw_report_file)

    pp.clean_file(test_file, [])
    pp.generate_report(arch.archive_name)
    arch.delete_archive_dir()

    if obfuscate:
        assert os.path.isfile(ip_report_file)
        with open(ip_report_file, 'r') as fp:
            ips = list(csv.reader(fp.readlines(), skipinitialspace=True))
            # ip
            assert len(ips) > 1
            assert ips[0] == ['Obfuscated IP', 'Original IP']
            assert ips[1] == ['10.230.230.1', '10.0.2.155']
        os.unlink(ip_report_file)
    else:
        assert not os.path.isfile(ip_report_file)

    if obfuscate_hostname:
        assert os.path.isfile(hn_report_file)
        with open(hn_report_file, 'r') as fp:
            hns = list(csv.reader(fp.readlines(), skipinitialspace=True))
            # hn
            assert len(hns) > 1
            assert hns[0] == ['Obfuscated Hostname', 'Original Hostname']
            assert hns[1] == ['f9fe0db0c046.example.com', 'report.test.com']
        os.unlink(hn_report_file)
    else:
        assert not os.path.isfile(hn_report_file)

    if rm_conf:
        assert os.path.isfile(kw_report_file)
        with open(kw_report_file, 'r') as fp:
            kws = list(csv.reader(fp.readlines(), skipinitialspace=True))
            # kw
            assert len(kws) > 1
            assert kws[0] == ['Replaced Keyword', 'Original Keyword']
            assert kws[1] == ['testword', 'keyword0']
        os.unlink(kw_report_file)
    else:
        assert not os.path.isfile(kw_report_file)



"""
PackageProvidesCommand - Command ``/bin/echo {command_package}``
================================================================
"""
from insights.core import CommandParser
from insights.core.exceptions import ParseException, SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.package_provides_command)
class PackageProvidesCommand(CommandParser, dict):
    """
    Parser to parse the specified running commands and its provider packages.

    This parser will receive a list of string pairs which is generated by
    several @datasource and functions. The first string is the full path of the
    specified running ``command`` and the second string is the package that
    provides this command.  It works as a `dict` with the ``command`` as the
    key and the corresponding package name as the value.

    To check the provider package of the specified command, please add the
    command to the ``COMMANDS`` of :func:`insights.specs.default.DefaultSpecs.cmd_and_pkg`

    Sample output::

        /usr/bin/java java-11-openjdk-11.0.9.11-2.el8_3.x86_64
        /usr/sbin/httpd httpd-2.4.22-7.el7.x86_64
        /usr/lib/jvm/jre/bin/java java-1.8.0-openjdk-1.8.0.272.b10-3.el8_3.x86_64
        /opt/rh/httpd24/root/usr/sbin/httpd httpd24-httpd-2.4.34-7.el7.x86_64

    Raises:
        SkipComponent: When no such command detected running on this host.
        ParseException: When there is un-parseble line.

    Example:
        >>> '/usr/lib/jvm/jre/bin/java' in cmd_package.commands
        True
        >>> 'java-11-openjdk-11.0.9.11-2.el8_3.x86_64' in cmd_package.packages
        True
        >>> '/usr/sbin/httpd' in cmd_package.commands
        True
        >>> 'httpd24-httpd-2.4.34-7.el7.x86_64' in cmd_package.packages
        True
        >>> cmd_package['/usr/lib/jvm/jre/bin/java']
        'java-1.8.0-openjdk-1.8.0.272.b10-3.el8_3.x86_64'
        >>> cmd_package['/usr/sbin/httpd']
        'httpd-2.4.22-7.el7.x86_64'
    """

    def parse_content(self, content):
        data = {}
        for line in content:
            l_sp = [l.strip() for l in line.split()]
            if len(l_sp) != 2:
                raise ParseException('Incorrect line: {0}'.format(line))
            data[l_sp[0]] = l_sp[1]

        if len(data) == 0:
            raise SkipComponent()

        self.update(data)

    @property
    def commands(self):
        """
        Returns the list of specified commands that are running on this host.
        """
        return list(self.keys())

    @property
    def packages(self):
        """
        Returns the list of the packages that provide the specified ``commands``.
        """
        return list(self.values())



"""
Ceph status commands
====================

This module provides processing for the output of the following ceph related
commands with ``-f json-pretty`` parameter.

CephOsdDump - command ``ceph osd dump -f json-pretty``
------------------------------------------------------

CephOsdDf - command ``ceph osd df -f json-pretty``
--------------------------------------------------

CephS - command ``ceph -s -f json-pretty``
------------------------------------------

CephDfDetail - command ``ceph df detail -f json-pretty``
--------------------------------------------------------

CephHealthDetail - command ``ceph health detail -f json-pretty``
--------------------------------------------------------------------------

CephECProfileGet - command ``ceph osd erasure-code-profile get default -f json-pretty``
---------------------------------------------------------------------------------------

CephCfgInfo - command ``ceph daemon {ceph_socket_files} config show``
---------------------------------------------------------------------

CephOsdTree - command ``ceph osd tree -f json-pretty``
------------------------------------------------------

CephReport - command ``ceph report``
------------------------------------

All these parsers are based on a shared class which processes the JSON
information into a dictionary.
"""
import json

from insights.core import CommandParser, JSONParser, LegacyItemAccess
from insights.core.exceptions import ParseException, SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.ceph_osd_dump)
class CephOsdDump(CommandParser, JSONParser):
    """
    Class to parse the output of ``ceph osd dump -f json-pretty``.

    Examples:

    >>> type(ceph_osd_dump)
    <class 'insights.parsers.ceph_cmd_json_parsing.CephOsdDump'>
    >>> ceph_osd_dump['pools'][0]['min_size']
    2
    """
    pass


@parser(Specs.ceph_osd_df)
class CephOsdDf(CommandParser, JSONParser):
    """
    Class to parse the output of ``ceph osd df -f json-pretty``.

    Examples:

    >>> type(ceph_osd_df)
    <class 'insights.parsers.ceph_cmd_json_parsing.CephOsdDf'>
    >>> ceph_osd_df['nodes'][0]['pgs']
    945
    """
    pass


@parser(Specs.ceph_s)
class CephS(CommandParser, JSONParser):
    """
    Class to parse the output of ``ceph -s -f json-pretty``.

    Examples:

    >>> type(ceph_s)
    <class 'insights.parsers.ceph_cmd_json_parsing.CephS'>
    >>> ceph_s['pgmap']['pgs_by_state'][0]['state_name'] == 'active+clean'
    True
    """
    pass


@parser(Specs.ceph_df_detail)
class CephDfDetail(CommandParser, JSONParser):
    """
    Class to parse the output of ``ceph df detail -f json-pretty``.

    Examples:

    >>> type(ceph_df_detail)
    <class 'insights.parsers.ceph_cmd_json_parsing.CephDfDetail'>
    >>> ceph_df_detail['stats']['total_avail_bytes']
    16910123008
    """
    pass


@parser(Specs.ceph_health_detail)
class CephHealthDetail(CommandParser, JSONParser):
    """
    Class to parse the output of ``ceph health detail -f json-pretty``.

    Examples:

    >>> type(ceph_health_detail)
    <class 'insights.parsers.ceph_cmd_json_parsing.CephHealthDetail'>
    >>> ceph_health_detail["overall_status"] ==   'HEALTH_OK'
    True
    """
    pass


@parser(Specs.ceph_osd_ec_profile_get)
class CephECProfileGet(CommandParser, JSONParser):
    """
    Class to parse the output of ``ceph osd erasure-code-profile get default -f json-pretty``.

    Examples:

    >>> type(ceph_osd_ec_profile_get)
    <class 'insights.parsers.ceph_cmd_json_parsing.CephECProfileGet'>
    >>> ceph_osd_ec_profile_get['k'] == '2'
    True
    """
    pass


@parser(Specs.ceph_config_show)
class CephCfgInfo(CommandParser, JSONParser):
    """
    Class to parse the output of ``ceph daemon .. config show``

    Examples:

    >>> type(ceph_cfg_info)
    <class 'insights.parsers.ceph_cmd_json_parsing.CephCfgInfo'>
    >>> ceph_cfg_info.max_open_files == '131072'
    True
    """
    @property
    def max_open_files(self):
        """
        str: Return the value of max_open_files
        """
        return self.data["max_open_files"]


@parser(Specs.ceph_osd_tree)
class CephOsdTree(CommandParser, JSONParser):
    """
    Class to parse the output of the command "ceph osd tree -f json-pretty

    Examples:

    >>> type(ceph_osd_tree)
    <class 'insights.parsers.ceph_cmd_json_parsing.CephOsdTree'>
    >>> ceph_osd_tree['nodes'][0]['children']
    [-5, -4, -3, -2]
    """
    pass


@parser(Specs.ceph_report)
class CephReport(CommandParser, LegacyItemAccess):
    """
    Class to parse the output of the command ``ceph report``.

    Examples:
    >>> type(ceph_report_content)
    <class 'insights.parsers.ceph_cmd_json_parsing.CephReport'>
    >>> ceph_report_content["version"] == '12.2.8-52.el7cp'
    True
    """
    def parse_content(self, content):
        if not content:
            raise SkipComponent("Empty output.")
        if not content[0].startswith('report ') or len(content) < 10:
            raise ParseException("Invalid Content: ", content[0])

        try:
            self.data = json.loads(''.join(content[1:]))
        except:
            raise ParseException("Could not parse json.", content[1:])



from insights.parsers.nmcli import NmcliConnShow
from insights.specs.datasources.nmcli import nmcli_conn_show_uuids
from insights.tests import context_wrap


NMCLI_CONN_SHOW = '''
NAME                UUID                                  TYPE      DEVICE
Wired connection 2  5fb06bd0-b09a-4573-b393-ffffffffffff  ethernet  enp0s20f0u5c2
lo                  10f69a0f-0bb0-409f-831f-dddddddddddd  loopback  lo
Wired connection 1  9dc5d4c5-71ae-44fb-804a-cccccccccccc  ethernet  --
enp0s29f2           bb30e099-8220-3eba-45f1-aaaaaaaaaaaa  ethernet  --
'''.strip()


def test_nmcli_conn_show_uuids():
    nmcli_conn_show = NmcliConnShow(context_wrap(NMCLI_CONN_SHOW))
    broker = {NmcliConnShow: nmcli_conn_show}
    result = nmcli_conn_show_uuids(broker)
    assert result == ['5fb06bd0-b09a-4573-b393-ffffffffffff', '10f69a0f-0bb0-409f-831f-dddddddddddd', '9dc5d4c5-71ae-44fb-804a-cccccccccccc', 'bb30e099-8220-3eba-45f1-aaaaaaaaaaaa']



import doctest

from insights.parsers import qemu_xml
from insights.tests import context_wrap

XML_NUMA = """
<!--
WARNING: THIS IS AN AUTO-GENERATED FILE. CHANGES TO IT ARE LIKELY TO BE
OVERWRITTEN AND LOST. Changes to this xml configuration should be made using:
  virsh edit 05-s00c06h0
or other application using the libvirt API.
-->

<domain type='kvm'>
  <name>05-s00c06h0</name>
  <uuid>02cf0bba-2bd6-11e7-8337-e4115b9a50d0</uuid>
  <memory unit='KiB'>12582912</memory>
  <currentMemory unit='KiB'>12582912</currentMemory>
  <vcpu placement='static'>4</vcpu>
  <cputune>
    <vcpupin vcpu='0' cpuset='1'/>
    <vcpupin vcpu='1' cpuset='2'/>
    <vcpupin vcpu='2' cpuset='3'/>
    <vcpupin vcpu='3' cpuset='4'/>
    <emulatorpin cpuset='1-4'/>
  </cputune>
  <numatune>
    <memory mode='strict' nodeset='0-1'/>
    <memnode cellid='0' mode='strict' nodeset='0'/>
    <memnode cellid='1' mode='strict' nodeset='1'/>
  </numatune>
  <os>
    <type arch='x86_64' machine='pc-i440fx-rhel7.0.0'>hvm</type>
    <boot dev='hd'/>
    <boot dev='network'/>
    <bootmenu enable='yes' timeout='1000'/>
    <bios useserial='yes' rebootTimeout='0'/>
  </os>
  <features>
    <acpi/>
    <apic/>
    <pae/>
  </features>
  <cpu>
    <numa>
      <cell id='0' cpus='0-1' memory='6291456' unit='KiB'/>
      <cell id='1' cpus='2-3' memory='6291456' unit='KiB'/>
    </numa>
  </cpu>
  <clock offset='utc'/>
  <on_poweroff>destroy</on_poweroff>
  <on_reboot>restart</on_reboot>
  <on_crash>restart</on_crash>
  <devices>
    <emulator>/usr/libexec/qemu-kvm</emulator>
    <disk type='file' device='disk'>
      <driver name='qemu' type='raw' cache='none' io='threads'/>
      <source file='/var/lib/libvirt/images/05-s00c06h0_1.img'/>
      <target dev='vda' bus='virtio'/>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x05' function='0x0'/>
    </disk>
    <controller type='usb' index='0'>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x01' function='0x2'/>
    </controller>
    <controller type='pci' index='0' model='pci-root'/>
    <controller type='virtio-serial' index='0'>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x03' function='0x0'/>
    </controller>
    <interface type='hostdev' managed='yes'>
      <mac address='aa:ss:bb:cc:dd:ee'/>
      <source>
        <address type='pci' domain='0x0000' bus='0x04' slot='0x10' function='0x0'/>
      </source>
      <rom bar='on' file='/opt/vcp/share/ipxe/808610ed.rom'/>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x04' function='0x0'/>
    </interface>
    <interface type='hostdev' managed='yes'>
      <mac address='bb:55:77:11:00:00'/>
      <source>
        <address type='pci' domain='0x0000' bus='0x04' slot='0x10' function='0x1'/>
      </source>
      <rom bar='on' file='/opt/vcp/share/ipxe/808610ed.rom'/>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x08' function='0x0'/>
    </interface>
    <serial type='pty'>
      <target port='0'/>
    </serial>
    <console type='pty'>
      <target type='serial' port='0'/>
    </console>
    <channel type='pipe'>
      <source path='/var/lib/libvirt/qemu/channels/FROM-05-s00c06h0'/>
      <target type='virtio' name='virtio2host'/>
      <address type='virtio-serial' controller='0' bus='0' port='1'/>
    </channel>
    <channel type='pipe'>
      <source path='/var/lib/libvirt/qemu/channels/HGC-05-s00c06h0'/>
      <target type='virtio' name='virtio_host_guest_check'/>
      <address type='virtio-serial' controller='0' bus='0' port='2'/>
    </channel>
    <input type='mouse' bus='ps2'/>
    <input type='keyboard' bus='ps2'/>
    <graphics type='vnc' port='-1' autoport='yes'>
      <listen type='address'/>
    </graphics>
    <video>
      <model type='cirrus' vram='16384' heads='1' primary='yes'/>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x02' function='0x0'/>
    </video>
    <watchdog model='i6300esb' action='reset'>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x07' function='0x0'/>
    </watchdog>
    <memballoon model='virtio'>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x06' function='0x0'/>
    </memballoon>
  </devices>
</domain>
"""

XML_NUMA_NO_NAME = """
<!--
WARNING: THIS IS AN AUTO-GENERATED FILE. CHANGES TO IT ARE LIKELY TO BE
OVERWRITTEN AND LOST. Changes to this xml configuration should be made using:
  virsh edit 05-s00c06h0
or other application using the libvirt API.
-->

<domain type='kvm'>
  <uuid>02cf0bba-2bd6-11e7-8337-e4115b9a50d0</uuid>
  <memory unit='KiB'>12582912</memory>
</domain>
"""

BLANK_XML = """
"""

RHEL_7_4_XML = """
<!--
WARNING: THIS IS AN AUTO-GENERATED FILE. CHANGES TO IT ARE LIKELY TO BE
OVERWRITTEN AND LOST. Changes to this xml configuration should be made using:
  virsh edit rhel7.4
or other application using the libvirt API.
-->

<domain type='kvm'>
  <name>rhel7.4</name>
  <uuid>b75610eb-06a0-4834-8286-e3d25d79c593</uuid>
  <memory unit='KiB'>4194304</memory>
  <currentMemory unit='KiB'>4194304</currentMemory>
  <vcpu placement='static'>4</vcpu>
  <os>
    <type arch='x86_64' machine='pc-i440fx-2.10'>hvm</type>
    <boot dev='hd'/>
  </os>
  <features>
    <acpi/>
    <apic/>
    <vmport state='off'/>
  </features>
  <cpu mode='custom' match='exact' check='partial'>
    <model fallback='allow'>Haswell-noTSX</model>
  </cpu>
  <clock offset='utc'>
    <timer name='rtc' tickpolicy='catchup'/>
    <timer name='pit' tickpolicy='delay'/>
    <timer name='hpet' present='no'/>
  </clock>
  <on_poweroff>destroy</on_poweroff>
  <on_reboot>restart</on_reboot>
  <on_crash>destroy</on_crash>
  <pm>
    <suspend-to-mem enabled='no'/>
    <suspend-to-disk enabled='no'/>
  </pm>
  <devices>
    <emulator>/usr/bin/qemu-kvm</emulator>
    <disk type='file' device='disk'>
      <driver name='qemu' type='qcow2'/>
      <source file='/var/lib/libvirt/images/rhel-7.4-insights.qcow2'/>
      <target dev='vda' bus='virtio'/>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x07' function='0x0'/>
    </disk>
    <controller type='usb' index='0' model='ich9-ehci1'>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x05' function='0x7'/>
    </controller>
    <controller type='usb' index='0' model='ich9-uhci1'>
      <master startport='0'/>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x05' function='0x0' multifunction='on'/>
    </controller>
    <controller type='usb' index='0' model='ich9-uhci2'>
      <master startport='2'/>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x05' function='0x1'/>
    </controller>
    <controller type='usb' index='0' model='ich9-uhci3'>
      <master startport='4'/>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x05' function='0x2'/>
    </controller>
    <controller type='pci' index='0' model='pci-root'/>
    <controller type='virtio-serial' index='0'>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x06' function='0x0'/>
    </controller>
    <interface type='network'>
      <mac address='aa:ss:bb:ff:ee:mm'/>
      <source network='default'/>
      <model type='virtio'/>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x03' function='0x0'/>
    </interface>
    <serial type='pty'>
      <target port='0'/>
    </serial>
    <console type='pty'>
      <target type='serial' port='0'/>
    </console>
    <channel type='unix'>
      <target type='virtio' name='org.qemu.guest_agent.0'/>
      <address type='virtio-serial' controller='0' bus='0' port='1'/>
    </channel>
    <channel type='spicevmc'>
      <target type='virtio' name='com.redhat.spice.0'/>
      <address type='virtio-serial' controller='0' bus='0' port='2'/>
    </channel>
    <input type='tablet' bus='usb'>
      <address type='usb' bus='0' port='1'/>
    </input>
    <input type='mouse' bus='ps2'/>
    <input type='keyboard' bus='ps2'/>
    <graphics type='spice' autoport='yes'>
      <listen type='address'/>
      <image compression='off'/>
    </graphics>
    <sound model='ich6'>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x04' function='0x0'/>
    </sound>
    <video>
      <model type='qxl' ram='65536' vram='65536' vgamem='16384' heads='1' primary='yes'/>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x02' function='0x0'/>
    </video>
    <redirdev bus='usb' type='spicevmc'>
      <address type='usb' bus='0' port='2'/>
    </redirdev>
    <redirdev bus='usb' type='spicevmc'>
      <address type='usb' bus='0' port='3'/>
    </redirdev>
    <memballoon model='virtio'>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x08' function='0x0'/>
    </memballoon>
  </devices>
</domain>
"""

XML_IDM_CLIENT = """
<!--
WARNING: THIS IS AN AUTO-GENERATED FILE. CHANGES TO IT ARE LIKELY TO BE
OVERWRITTEN AND LOST. Changes to this xml configuration should be made using:
  virsh edit test-idm-client-ccveu-net
or other application using the libvirt API.
-->

<domstatus state='running' reason='unpaused' pid='17150'>
  <monitor path='/var/lib/libvirt/qemu/domain-59-test-idm-client-ccve/monitor.sock' json='1' type='unix'/>
  <vcpus>
    <vcpu id='0' pid='17156'/>
    <vcpu id='1' pid='17157'/>
  </vcpus>
  <qemuCaps>
    <flag name='kvm'/>
    <flag name='mem-path'/>
    <flag name='ivshmem-doorbell'/>
  </qemuCaps>
  <devices>
    <device alias='balloon0'/>
    <device alias='virtio-disk0'/>
    <device alias='ide0-1-0'/>
  </devices>
  <libDir path='/var/lib/libvirt/qemu/domain-59-test-idm-client-ccve'/>
  <channelTargetDir path='/var/lib/libvirt/qemu/channel/target/domain-59-test-idm-client-ccve'/>
  <domain type='kvm' id='59'>
    <name>test-idm-client-ccveu-net</name>
    <uuid>78177d07-ac0e-4057-b1de-9ccd66cbc3d7</uuid>
    <metadata xmlns:ovirt="http://ovirt.org/vm/tune/1.0">
      <ovirt:qos/>
    </metadata>
    <maxMemory slots='16' unit='KiB'>4294967296</maxMemory>
    <memory unit='KiB'>2097152</memory>
    <currentMemory unit='KiB'>2097152</currentMemory>
    <vcpu placement='static' current='2'>32</vcpu>
    <cputune>
      <shares>1020</shares>
    </cputune>
    <resource>
      <partition>/machine</partition>
    </resource>
    <sysinfo type='smbios'>
      <system>
        <entry name='manufacturer'>Red Hat</entry>
        <entry name='product'>RHEV Hypervisor</entry>
        <entry name='version'>7.3-1.1.el7</entry>
        <entry name='serial'>49c1e6bb-adbb-44ac-8d12-5ba4119cf110</entry>
        <entry name='uuid'>78177d07-ac0e-4057-b1de-9ccd66cbc3d7</entry>
      </system>
    </sysinfo>
    <os>
      <type arch='x86_64' machine='pc-i440fx-rhel7.2.0'>hvm</type>
      <bootmenu enable='yes' timeout='10000'/>
      <smbios mode='sysinfo'/>
    </os>
    <features>
      <acpi/>
    </features>
    <cpu mode='custom' match='exact'>
      <model fallback='allow'>Broadwell</model>
      <topology sockets='16' cores='2' threads='1'/>
      <numa>
        <cell id='0' cpus='0-1' memory='2097152' unit='KiB'/>
      </numa>
    </cpu>
    <clock offset='variable' adjustment='1' basis='utc'>
      <timer name='rtc' tickpolicy='catchup'/>
      <timer name='pit' tickpolicy='delay'/>
      <timer name='hpet' present='no'/>
    </clock>
    <on_poweroff>destroy</on_poweroff>
    <on_reboot>restart</on_reboot>
    <on_crash>destroy</on_crash>
    <devices>
      <emulator>/usr/libexec/qemu-kvm</emulator>
      <disk type='file' device='cdrom'>
        <driver name='qemu' type='raw'/>
        <source startupPolicy='optional'/>
        <backingStore/>
        <target dev='hdc' bus='ide'/>
        <readonly/>
        <alias name='ide0-1-0'/>
        <address type='drive' controller='0' bus='1' target='0' unit='0'/>
      </disk>
      <controller type='usb' index='0' model='piix3-uhci'>
        <alias name='usb'/>
        <address type='pci' domain='0x0000' bus='0x00' slot='0x01' function='0x2'/>
      </controller>
      <interface type='bridge'>
        <mac address='00:1a:4a:16:03:72'/>
        <source bridge='vlan2593'/>
        <target dev='vnet20'/>
        <model type='virtio'/>
        <filterref filter='vdsm-no-mac-spoofing'/>
        <link state='up'/>
        <boot order='2'/>
        <alias name='net0'/>
        <address type='pci' domain='0x0000' bus='0x00' slot='0x03' function='0x0'/>
      </interface>
      <channel type='unix'>
        <source mode='bind' path='/var/lib/libvirt/qemu/channels/78177d07-ac0e-4057-b1de-9ccd66cbc3d7.com.redhat.rhevm.vdsm'/>
        <target type='virtio' name='com.redhat.rhevm.vdsm' state='connected'/>
        <alias name='channel0'/>
        <address type='virtio-serial' controller='0' bus='0' port='1'/>
      </channel>
      <input type='mouse' bus='ps2'>
        <alias name='input0'/>
      </input>
    </devices>
    <seclabel type='dynamic' model='dac' relabel='yes'>
      <label>+107:+107</label>
      <imagelabel>+107:+107</imagelabel>
    </seclabel>
  </domain>
</domstatus>
""".strip()

XML_IDM_CLIENT_NO_NAME = """
<!--
WARNING: THIS IS AN AUTO-GENERATED FILE. CHANGES TO IT ARE LIKELY TO BE
OVERWRITTEN AND LOST. Changes to this xml configuration should be made using:
  virsh edit test-idm-client-ccveu-net
or other application using the libvirt API.
-->

<domstatus state='running' reason='unpaused' pid='17150'>
  <monitor path='/var/lib/libvirt/qemu/domain-59-test-idm-client-ccve/monitor.sock' json='1' type='unix'/>
  <vcpus>
    <vcpu id='0' pid='17156'/>
    <vcpu id='1' pid='17157'/>
  </vcpus>
  <libDir path='/var/lib/libvirt/qemu/domain-59-test-idm-client-ccve'/>
  <channelTargetDir path='/var/lib/libvirt/qemu/channel/target/domain-59-test-idm-client-ccve'/>
  <domain type='kvm' id='59'>
    <uuid>78177d07-ac0e-4057-b1de-9ccd66cbc3d7</uuid>
  </domain>
</domstatus>
""".strip()

XML_RHOSP = """
<!--
WARNING: THIS IS AN AUTO-GENERATED FILE. CHANGES TO IT ARE LIKELY TO BE
OVERWRITTEN AND LOST. Changes to this xml configuration should be made using:
  virsh edit instance-000008d6
or other application using the libvirt API.
-->

<domain type='kvm'>
  <name>instance-000008d6</name>
  <uuid>601da52c-450f-4d8e-b502-503431536fa69</uuid>
  <metadata>
    <nova:instance xmlns:nova="http://openstack.org/xmlns/libvirt/nova/1.0">
      <nova:package version="14.0.3-8.el7ost"/>
      <nova:name>django_vm_001</nova:name>
      <nova:creationTime>2017-10-09 08:51:28</nova:creationTime>
      <nova:flavor name="vpc1-cf1-foo-bar">
        <nova:memory>8096</nova:memory>
        <nova:disk>10</nova:disk>
        <nova:swap>0</nova:swap>
        <nova:ephemeral>0</nova:ephemeral>
        <nova:vcpus>4</nova:vcpus>
      </nova:flavor>
      <nova:owner>
        <nova:user uuid="96e9d2b749ea48fcb5a911e6f0e144f2">django_user_01</nova:user>
        <nova:project uuid="5a50e9d0d19746158958be0c759793fb">vpcdi1</nova:project>
      </nova:owner>
      <nova:root type="image" uuid="1a05a423-dfae-428a-ae54-1614d8024e76"/>
    </nova:instance>
  </metadata>
  <memory unit='KiB'>8290304</memory>
  <currentMemory unit='KiB'>8290304</currentMemory>
  <vcpu placement='static'>4</vcpu>
  <cputune>
    <shares>4096</shares>
    <vcpupin vcpu='0' cpuset='1'/>
    <vcpupin vcpu='1' cpuset='2'/>
    <vcpupin vcpu='2' cpuset='13'/>
    <vcpupin vcpu='3' cpuset='14'/>
    <emulatorpin cpuset='1-2,13-14'/>
  </cputune>
  <numatune>
    <memory mode='strict' nodeset='0-1'/>
    <memnode cellid='0' mode='strict' nodeset='0'/>
    <memnode cellid='1' mode='strict' nodeset='1'/>
  </numatune>
  <sysinfo type='smbios'>
    <system>
      <entry name='manufacturer'>Red Hat</entry>
      <entry name='product'>OpenStack Compute</entry>
      <entry name='version'>14.0.3-8.el7ost</entry>
      <entry name='serial'>1b52c42f-b1eb-47f4-a8c0-9904d4735fc9</entry>
      <entry name='uuid'>601da52c-450f-4d8e-b502-50151536fa69</entry>
      <entry name='family'>Virtual Machine</entry>
    </system>
  </sysinfo>
  <os>
    <type arch='x86_64' machine='pc-i440fx-rhel7.3.0'>hvm</type>
    <boot dev='hd'/>
    <smbios mode='sysinfo'/>
  </os>
  <features>
    <acpi/>
    <apic/>
  </features>
  <cpu mode='host-passthrough'>
    <topology sockets='4' cores='1' threads='1'/>
    <numa>
      <cell id='0' cpus='0-1' memory='4145152' unit='KiB'/>
      <cell id='1' cpus='2-3' memory='4145152' unit='KiB'/>
    </numa>
  </cpu>
  <clock offset='utc'>
    <timer name='pit' tickpolicy='delay'/>
    <timer name='rtc' tickpolicy='catchup'/>
    <timer name='hpet' present='no'/>
  </clock>
  <on_poweroff>destroy</on_poweroff>
  <on_reboot>restart</on_reboot>
  <on_crash>destroy</on_crash>
  <devices>
    <emulator>/usr/libexec/qemu-kvm</emulator>
    <disk type='file' device='disk'>
      <driver name='qemu' type='qcow2' cache='none'/>
      <source file='/var/lib/nova/instances/601da52c-450f-4d8e-b502-50151536fa69/disk'/>
      <target dev='vda' bus='virtio'/>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x04' function='0x0'/>
    </disk>
    <disk type='block' device='disk'>
      <driver name='qemu' type='raw' cache='none' io='native'/>
      <source dev='/dev/disk/by-id/dm-uuid-mpath-3600a09803830316d512b4a6b38374143'/>
      <target dev='vdb' bus='virtio'/>
      <serial>dd036ba1-bb5f-4c1a-a369-a918080bff7f</serial>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x05' function='0x0'/>
    </disk>
    <controller type='usb' index='0'>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x01' function='0x2'/>
    </controller>
    <controller type='pci' index='0' model='pci-root'/>
    <interface type='bridge'>
      <mac address='aa:aa:ee:bb:10:aa'/>
      <source bridge='qbrf975e53f-e0'/>
      <target dev='tapf975e53f-e0'/>
      <model type='virtio'/>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x03' function='0x0'/>
    </interface>
    <serial type='file'>
      <source path='/var/lib/nova/instances/601da52c-450f-4d8e-b502-50151536fa69/console.log'/>
      <target port='0'/>
    </serial>
    <serial type='pty'>
      <target port='1'/>
    </serial>
    <console type='file'>
      <source path='/var/lib/nova/instances/601da52c-450f-4d8e-b502-50151536fa69/console.log'/>
      <target type='serial' port='0'/>
    </console>
    <input type='tablet' bus='usb'>
      <address type='usb' bus='0' port='1'/>
    </input>
    <input type='mouse' bus='ps2'/>
    <input type='keyboard' bus='ps2'/>
    <graphics type='vnc' port='-1' autoport='yes' listen='0.0.0.0' keymap='fr'>
      <listen type='address' address='0.0.0.0'/>
    </graphics>
    <video>
      <model type='cirrus' vram='16384' heads='1' primary='yes'/>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x02' function='0x0'/>
    </video>
    <memballoon model='virtio'>
      <stats period='10'/>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x06' function='0x0'/>
    </memballoon>
  </devices>
</domain>
""".strip()

XML_RHOSP_2 = """
<!--
WARNING: THIS IS AN AUTO-GENERATED FILE. CHANGES TO IT ARE LIKELY TO BE
OVERWRITTEN AND LOST. Changes to this xml configuration should be made using:
  virsh edit instance-000008d6
or other application using the libvirt API.
-->

<domain type='kvm'>
  <name>instance-000008d7</name>
  <uuid>601da52c-450f-4d8e-b502-503431536fa69</uuid>
  <metadata>
    <nova:instance xmlns:nova="http://openstack.org/xmlns/libvirt/nova/1.0">
      <nova:package version="14.0.3-8.el7ost"/>
      <nova:name>VM_AT_01</nova:name>
      <nova:creationTime>2017-10-09 08:51:28</nova:creationTime>
    </nova:instance>
  </metadata>
  <memory unit='KiB'>8290304</memory>
  <currentMemory unit='KiB'>8290304</currentMemory>
  <vcpu placement='static'>4</vcpu>
  <cputune>
    <shares>4096</shares>
    <vcpupin vcpu='0' cpuset='1'/>
    <vcpupin vcpu='1' cpuset='2'/>
    <vcpupin vcpu='2' cpuset='13'/>
    <vcpupin vcpu='3' cpuset='14'/>
    <emulatorpin cpuset='1-2,13-14'/>
  </cputune>
  <numatune>
    <memory mode='strict' nodeset='0-1'/>
    <memnode cellid='0' mode='strict' nodeset='0'/>
    <memnode cellid='1' mode='strict' nodeset='1'/>
  </numatune>
  <sysinfo type='smbios'>
    <system>
      <entry name='manufacturer'>Red Hat</entry>
      <entry name='product'>OpenStack Compute</entry>
      <entry name='version'>14.0.3-8.el7ost</entry>
      <entry name='serial'>1b52c42f-b1eb-47f4-a8c0-9904d4735fc9</entry>
      <entry name='uuid'>601da52c-450f-4d8e-b502-50151536fa69</entry>
      <entry name='family'>Virtual Machine</entry>
    </system>
  </sysinfo>
  <os>
    <type arch='x86_64' machine='pc-i440fx-rhel7.3.0'>hvm</type>
    <boot dev='hd'/>
    <smbios mode='sysinfo'/>
  </os>
  <features>
    <acpi/>
    <apic/>
  </features>
  <cpu mode='host-passthrough'>
    <topology sockets='4' cores='1' threads='1'/>
    <numa>
      <cell id='0' cpus='0-1' memory='4145152' unit='KiB'/>
      <cell id='1' cpus='2-3' memory='4145152' unit='KiB'/>
    </numa>
  </cpu>
  <clock offset='utc'>
    <timer name='pit' tickpolicy='delay'/>
    <timer name='rtc' tickpolicy='catchup'/>
    <timer name='hpet' present='no'/>
  </clock>
  <on_poweroff>destroy</on_poweroff>
  <on_reboot>restart</on_reboot>
  <on_crash>destroy</on_crash>
  <devices>
    <emulator>/usr/libexec/qemu-kvm</emulator>
    <disk type='file' device='disk'>
      <driver name='qemu' type='qcow2' cache='none'/>
      <source file='/var/lib/nova/instances/601da52c-450f-4d8e-b502-50151536fa69/disk'/>
      <target dev='vda' bus='virtio'/>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x04' function='0x0'/>
    </disk>
    <disk type='block' device='disk'>
      <driver name='qemu' type='raw' cache='none' io='native'/>
      <source dev='/dev/disk/by-id/dm-uuid-mpath-3600a09803830316d512b4a6b38374143'/>
      <target dev='vdb' bus='virtio'/>
      <serial>dd036ba1-bb5f-4c1a-a369-a918080bff7f</serial>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x05' function='0x0'/>
    </disk>
    <controller type='usb' index='0'>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x01' function='0x2'/>
    </controller>
    <controller type='pci' index='0' model='pci-root'/>
    <interface type='bridge'>
      <mac address='aa:aa:ee:bb:10:aa'/>
      <source bridge='qbrf975e53f-e0'/>
      <target dev='tapf975e53f-e0'/>
      <model type='virtio'/>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x03' function='0x0'/>
    </interface>
    <serial type='file'>
      <source path='/var/lib/nova/instances/601da52c-450f-4d8e-b502-50151536fa69/console.log'/>
      <target port='0'/>
    </serial>
    <serial type='pty'>
      <target port='1'/>
    </serial>
    <console type='file'>
      <source path='/var/lib/nova/instances/601da52c-450f-4d8e-b502-50151536fa69/console.log'/>
      <target type='serial' port='0'/>
    </console>
    <input type='tablet' bus='usb'>
      <address type='usb' bus='0' port='1'/>
    </input>
    <input type='mouse' bus='ps2'/>
    <input type='keyboard' bus='ps2'/>
    <graphics type='vnc' port='-1' autoport='yes' listen='0.0.0.0' keymap='fr'>
      <listen type='address' address='0.0.0.0'/>
    </graphics>
    <video>
      <model type='cirrus' vram='16384' heads='1' primary='yes'/>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x02' function='0x0'/>
    </video>
    <memballoon model='virtio'>
      <stats period='10'/>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x06' function='0x0'/>
    </memballoon>
  </devices>
</domain>
""".strip()


def test_vm_xml():
    xml = qemu_xml.QemuXML(context_wrap(XML_NUMA, path='/etc/libvirt/qemu/vm.xml'))
    assert xml.file_name == 'vm.xml'
    assert xml.vm_name == '05-s00c06h0'
    memnode = xml.get_elements('./numatune/memnode', None)
    assert len(memnode[0].items()) == 3
    assert len(memnode[1].items()) == 3

    # No 'name' found
    xml = qemu_xml.QemuXML(context_wrap(XML_NUMA_NO_NAME, path='/etc/libvirt/qemu/no_name.xml'))
    assert xml.vm_name is None


def test_rhel_7_4():
    xml = qemu_xml.QemuXML(context_wrap(RHEL_7_4_XML, path='/etc/libvirt/qemu/rhel7.4.xml'))
    assert xml.file_name == 'rhel7.4.xml'
    assert xml.vm_name == 'rhel7.4'
    os = xml.get_elements('./os/type')[0]
    assert os.get('arch') == 'x86_64'
    assert os.get('machine') == 'pc-i440fx-2.10'


def test_parse_dom():
    xml = qemu_xml.QemuXML(context_wrap(RHEL_7_4_XML, path='/etc/libvirt/qemu/rhel7.4.xml'))
    dom = xml.parse_dom()
    assert dom.get('vcpu', None) == '4'

    xml = qemu_xml.QemuXML(context_wrap(XML_NUMA_NO_NAME, path='/etc/libvirt/qemu/no_name.xml'))
    dom = xml.parse_dom()
    assert dom.get('vcpu') is None


def test_blank_xml():
    xml = qemu_xml.QemuXML(context_wrap(BLANK_XML, path='/etc/libvirt/qemu/blank.xml'))
    assert xml.file_name == 'blank.xml'
    assert xml.vm_name is None
    assert xml.parse_dom() == {}


def test_rhosp_xml():
    xml = qemu_xml.OpenStackInstanceXML(context_wrap(XML_RHOSP, path='/etc/libvirt/qemu/instance-000008d6.xml'))
    assert xml.domain_name == 'instance-000008d6'
    assert xml.nova.get('version') == '14.0.3-8.el7ost'
    assert xml.nova.get('instance_name') == 'django_vm_001'
    assert xml.nova.get('user') == 'django_user_01'
    assert xml.nova.get('root_disk_type') == 'image'
    assert xml.nova.get('flavor_vcpus') == '4'

    rhosp_xml_2 = qemu_xml.OpenStackInstanceXML(context_wrap(XML_RHOSP_2, path='/etc/libvirt/qemu/instance-000008d7.xml'))
    assert rhosp_xml_2.nova == {'version': '14.0.3-8.el7ost', 'instance_name': 'VM_AT_01', 'created': '2017-10-09 08:51:28'}
    assert rhosp_xml_2.domain_name == 'instance-000008d7'

    # XML not having OSP metadata
    xml = qemu_xml.OpenStackInstanceXML(context_wrap(XML_NUMA, path='/etc/libvirt/qemu/numa_instance_001.xml'))
    assert xml.domain_name == 'numa_instance_001'

    # The XML struture is different from OSP instance XML
    xml = qemu_xml.OpenStackInstanceXML(context_wrap(XML_IDM_CLIENT, path='/etc/libvirt/qemu/idm_001.xml'))
    assert xml.domain_name == 'idm_001'


def test_documentation():
    env = {'xml_numa': qemu_xml.QemuXML(context_wrap(XML_NUMA, path='/etc/libvirt/qemu/vm.xml')),
           'rhosp_xml': qemu_xml.OpenStackInstanceXML(context_wrap(XML_RHOSP, path='/etc/libvirt/qemu/instance-000008d6.xml'))}
    failed_count, tests = doctest.testmod(qemu_xml, globs=env)
    assert failed_count == 0


def test_var_qemu_xml():
    xml = qemu_xml.VarQemuXML(context_wrap(XML_IDM_CLIENT, path='/var/run/libvirt/qemu/test-idm-client-ccveu-net.xml'))
    assert xml.file_name == 'test-idm-client-ccveu-net.xml'
    assert xml.vm_name == 'test-idm-client-ccveu-net'
    assert len(xml.get("qemuCaps")) == 3
    disks = xml.get_elements('./domain/devices/disk')
    assert len(disks) == 1
    assert disks[0].get('device', None) == 'cdrom'

    # No 'name' found
    xml = qemu_xml.VarQemuXML(context_wrap(XML_IDM_CLIENT_NO_NAME, path='/var/run/libvirt/qemu/no_name.xml'))
    assert xml.vm_name is None



import operator

from functools import reduce
from insights import datasource, dr, rule, make_info
from insights.core.spec_factory import DatasourceProvider, find, RegistryPoint, SpecSet


class Specs(SpecSet):
    the_data = RegistryPoint(filterable=True)


class MySpecs(Specs):

    @datasource()
    def the_data(broker):
        data = """
        foo bar baz
        baz bar
        """.strip()
        return DatasourceProvider(data, "the_data")


foos = find(Specs.the_data, "foo")
direct_foos = find(MySpecs.the_data, "foo")


@rule(foos, direct_foos)
def report(f, d):
    all_foos = reduce(operator.__iadd__, f.values(), [])
    all_direct_foos = reduce(operator.__iadd__, d.values(), [])
    return make_info("FOO", num_all_foos=len(all_foos), num_direct_foos=len(all_direct_foos))


def test_find():
    broker = dr.run(report)
    results = broker[report]
    assert "num_all_foos" in results
    assert "num_direct_foos" in results



"""
Custom datasource to get ssl certificate file path.
"""

from insights.combiners.rsyslog_confs import RsyslogAllConf
from insights.core.context import HostContext
from insights.core.exceptions import SkipComponent
from insights.core.plugins import datasource
from insights.parsers.mssql_conf import MsSQLConf
from insights.specs.datasources import httpd
from insights.specs import Specs


@datasource(httpd.httpd_configuration_files, HostContext)
def httpd_certificate_info_in_nss(broker):
    """
    Get the certificate info configured in nss database

    Arguments:
        broker: the broker object for the current session

    Returns:
        list: Returns a list of tuple with the Nss database path and the certificate nickname

    Raises:
        SkipComponent: Raised when NSSEngine isn't enabled or "NSSCertificateDatabase" and
            "NSSNickname" directives aren't found
    """
    confs = broker[httpd.httpd_configuration_files]
    path_pairs = []
    nss_engine = nss_database = cert_name = None
    virtual_host_start = False
    for conf in confs:
        with open(conf) as a_f:
            for line in a_f.readlines():
                line = line.strip()
                if line.startswith('<VirtualHost'):
                    virtual_host_start = True
                    continue
                if virtual_host_start:
                    if line.startswith('NSSEngine'):
                        nss_engine = line.split()[-1].lower().strip('"\'')
                    elif line.startswith('NSSCertificateDatabase'):
                        nss_database = line.split()[-1].lower().strip('"\'')
                    elif line.startswith('NSSNickname'):
                        cert_name = line.split()[-1].lower().strip('"\'')
                    elif line.startswith('</VirtualHost>'):
                        if nss_engine == 'on' and nss_database and cert_name:
                            path_pairs.append((nss_database, cert_name))
                        virtual_host_start = False
                        nss_engine = nss_database = cert_name = None
    if path_pairs:
        return path_pairs
    raise SkipComponent


@datasource(httpd.httpd_configuration_files, HostContext)
def httpd_ssl_certificate_files(broker):
    """
    Get the httpd SSL certificate file path configured by "SSLCertificateFile"

    Arguments:
        broker: the broker object for the current session

    Returns:
        str: Returns the SSL certificate file path configured by "SSLCertificateFile"

    Raises:
        SkipComponent: Raised if "SSLCertificateFile" directive isn't found
    """
    confs = broker[httpd.httpd_configuration_files]
    ssl_engine = ssl_cert = None
    ssl_certs = set()
    virtual_host_start = False
    for conf in confs:
        with open(conf) as a_f:
            for line in a_f.readlines():
                line = line.strip()
                if line.startswith('<VirtualHost'):
                    virtual_host_start = True
                    continue
                if virtual_host_start:
                    if line.startswith('SSLEngine'):
                        ssl_engine = line.split()[-1].lower().strip('"\'')
                    elif line.startswith('SSLCertificateFile'):
                        ssl_cert = line.strip().split()[-1].strip('"\'')
                    elif line.startswith('</VirtualHost>'):
                        if ssl_engine == 'on' and ssl_cert:
                            ssl_certs.add(ssl_cert)
                        virtual_host_start = False
                        ssl_engine = ssl_cert = None
    if ssl_certs:
        return sorted(ssl_certs)
    raise SkipComponent


@datasource(Specs.nginx_conf, HostContext)
def nginx_ssl_certificate_files(broker):
    """
    Get the nginx SSL certificate file path configured by "ssl_certificate"

    Arguments:
        broker: the broker object for the current session

    Returns:
        str: Returns the SSL certificate file path configured by "ssl_certificate"

    Raises:
        SkipComponent: Raised if "ssl_certificate" directive isn't found
    """
    ssl_certs = set()
    confs = broker[Specs.nginx_conf]
    for conf in confs:
        for line in conf.content:
            if 'ssl_certificate' in line:
                items = [item.strip('"\'; ') for item in line.split(None, 1)]
                if items[0] == 'ssl_certificate':
                    ssl_certs.add(items[1])
    if ssl_certs:
        return sorted(ssl_certs)
    raise SkipComponent


@datasource(MsSQLConf, HostContext)
def mssql_tls_cert_file(broker):
    """
    Get the mssql tls certificate file path configured by "ssl_certificate"

    Arguments:
        broker: the broker object for the current session
    Returns:
        str: Returns the SSL certificate file path configured by "ssl_certificate"
    Raises:
        SkipComponent: Raised if "ssl_certificate" directive isn't found
    """
    mssql_conf_content = broker[MsSQLConf]
    if mssql_conf_content.has_option("network", "tlscert"):
        return mssql_conf_content.get("network", "tlscert")
    raise SkipComponent


@datasource(RsyslogAllConf, HostContext)
def rsyslog_tls_cert_file(broker):
    """
    Get the rsyslog tls certificate file path configured by "DefaultNetstreamDriverCertFile"

    Arguments:
        broker: the broker object for the current session
    Returns:
        str: Returns the SSL certificate file path configured by "DefaultNetstreamDriverCertFile"
    Raises:
        SkipComponent: Raised if "DefaultNetstreamDriverCertFile" isn't found
    """
    rsyslog_objs = broker[RsyslogAllConf]
    for obj in rsyslog_objs.values():
        for item in obj:
            if 'DefaultNetstreamDriverCertFile' in item:
                if '$DefaultNetstreamDriverCertFile' in item:
                    # basic format
                    return item.split()[-1].strip()
                else:
                    # advanced format
                    # it is set in global block, and the global line contains all the content in it
                    parts = item.split()
                    for part in parts:
                        if 'DefaultNetstreamDriverCertFile' in part:
                            return part.split('=')[-1].strip('"')
    raise SkipComponent


@datasource(RsyslogAllConf, HostContext)
def rsyslog_tls_ca_cert_file(broker):
    """
    Get the rsyslog tls ca certificate file path configured by "DefaultNetstreamDriverCAFile"

    Arguments:
        broker: the broker object for the current session
    Returns:
        str: Returns the SSL certificate file path configured by "DefaultNetstreamDriverCAFile"
    Raises:
        SkipComponent: Raised if "DefaultNetstreamDriverCAFile" isn't found
    """
    rsyslog_objs = broker[RsyslogAllConf]
    for obj in rsyslog_objs.values():
        for item in obj:
            if 'DefaultNetstreamDriverCAFile' in item:
                if '$DefaultNetstreamDriverCAFile' in item:
                    # basic format
                    return item.split()[-1].strip()
                else:
                    # advanced format
                    # it is set in global block, and the global line contains all the content in it
                    parts = item.split()
                    for part in parts:
                        if 'DefaultNetstreamDriverCAFile' in part:
                            # example: global(DefaultNetstreamDriverCAFile="file_path" test="abc")
                            if '=' in part:
                                return part.split('=')[-1].strip('")')
                            else:
                                # example: global( DefaultNetstreamDriverCAFile = "file_path" test = "abc")
                                name_index = parts.index(part)
                                if len(parts) > name_index + 2:
                                    if parts[name_index + 1].strip() == '=':
                                        path = parts[name_index + 2].strip('")')
                                        if path:
                                            return path
                                # no need to continue
                                raise SkipComponent
    raise SkipComponent



import doctest
from insights.parsers import cloud_init_log
from insights.tests import context_wrap


CLOUD_INIT_LOG = """
2019-08-07 14:33:27,269 - util.py[DEBUG]: Reading from /etc/cloud/cloud.cfg.d/99-datasource.cfg (quiet=False)
2019-08-07 14:33:27,269 - util.py[DEBUG]: Read 59 bytes from /etc/cloud/cloud.cfg.d/99-datasource.cfg
2019-08-07 14:33:27,269 - util.py[DEBUG]: Attempting to load yaml from string of length 59 with allowed root types (<type 'dict'>,)
2019-08-07 14:33:27,270 - util.py[WARNING]: Failed loading yaml blob. Invalid format at line 1 column 1: "while parsing a block mapping
""".strip()


def test_cloud_init_log():
    log = cloud_init_log.CloudInitLog(context_wrap(CLOUD_INIT_LOG))
    assert "Reading from /etc/cloud/cloud.cfg.d/99-datasource.cfg" in log
    assert len(log.get('DEBUG')) == 3


def test_documentation():
    failed_count, tests = doctest.testmod(
        cloud_init_log,
        globs={'log': cloud_init_log.CloudInitLog(context_wrap(CLOUD_INIT_LOG))}
    )
    assert failed_count == 0



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import securetty
from insights.parsers.securetty import Securetty
from insights.tests import context_wrap

SECURETTY = """
console
# tty0
tty1
tty2
tty3
""".strip()

SECURETTY_EMPTY = ""


def test_securetty():
    securetty = Securetty(context_wrap(SECURETTY))
    assert len(securetty.terminals) == 4
    assert securetty.terminals == ['console', 'tty1', 'tty2', 'tty3']


def test_class_exceptions():
    with pytest.raises(SkipComponent) as e:
        Securetty(context_wrap(SECURETTY_EMPTY))
    assert "No terminals found." in str(e)


def test_doc():
    env = {
        "securetty": Securetty(context_wrap(SECURETTY)),
    }
    failed_count, total = doctest.testmod(securetty, globs=env)
    assert failed_count == 0



# -*- coding: utf-8 -*-

import pytest
try:
    from unittest.mock import patch, mock_open
    builtin_open = "builtins.open"
except Exception:
    from mock import patch, mock_open
    builtin_open = "__builtin__.open"

from insights.combiners.rsyslog_confs import RsyslogAllConf
from insights.core.exceptions import SkipComponent
from insights.parsers.mssql_conf import MsSQLConf
from insights.parsers.rsyslog_conf import RsyslogConf
from insights.specs import Specs
from insights.specs.datasources import httpd
from insights.specs.datasources.ssl_certificate import (
    httpd_ssl_certificate_files, nginx_ssl_certificate_files,
    mssql_tls_cert_file, httpd_certificate_info_in_nss,
    rsyslog_tls_cert_file, rsyslog_tls_ca_cert_file
)
from insights.tests import context_wrap


HTTPD_CONF = """
listen 80
listen 443
IncludeOptional "/etc/httpd/conf.d/*.conf"
""".strip()

HTTPD_SSL_CONF = """
<VirtualHost *:443>
  ## SSL directives
  SSLEngine on
  SSLCertificateFile      "/etc/pki/katello/certs/gént-katello-apache.crt"
  SSLCertificateKeyFile   "/etc/pki/katello/private/katello-apache.key"
  SSLCertificateChainFile "/etc/pki/katello/certs/katello-server-ca.crt"
  SSLVerifyClient         optional
  SSLCACertificateFile    "/etc/pki/katello/certs/katello-default-ca.crt"
  SSLVerifyDepth          3
  SSLOptions +StdEnvVars +ExportCertData
</VirtualHost>
""".strip()

HTTPD_SSL_CONF_2 = """
<VirtualHost *:443>
  ## SSL directives
  ServerName a.b.c.com
  SSLEngine on
  # SSLCertificateFile    "/etc/pki/katello/certs/old_katello-apache.crt"
  SSLCertificateFile      "/etc/pki/katello/certs/katello-apache.crt"
  SSLCertificateKeyFile   "/etc/pki/katello/private/katello-apache.key"
  SSLCertificateChainFile "/etc/pki/katello/certs/katello-server-ca.crt"
  SSLVerifyClient         optional
  SSLCACertificateFile    "/etc/pki/katello/certs/katello-default-ca.crt"
  SSLVerifyDepth          3
  SSLOptions +StdEnvVars +ExportCertData
</VirtualHost>
<VirtualHost *:443>
  ## SSL directives
  ServerName  d.c.e.com
  SSLEngine on
  # bellow is SSLCertificateFile configuration
  SSLCertificateFile      "/etc/pki/katello/certs/katello-apache_d.crt"
  SSLCertificateKeyFile   "/etc/pki/katello/private/katello-apache_d.key"
  SSLCertificateChainFile "/etc/pki/katello/certs/katello-server-ca_d.crt"
  SSLVerifyClient         optional
  SSLCACertificateFile    "/etc/pki/katello/certs/katello-default-ca_d.crt"
  SSLVerifyDepth          3
  SSLOptions +StdEnvVars +ExportCertData
</VirtualHost>
<VirtualHost *:443>
  ## SSL directives
  ServerName  f.g.e.com
  SSLEngine off
  SSLCertificateFile      "/etc/pki/katello/certs/katello-apache_e.crt"
  SSLCertificateKeyFile   "/etc/pki/katello/private/katello-apache_e.key"
  SSLCertificateChainFile "/etc/pki/katello/certs/katello-server-ca_e.crt"
  SSLVerifyClient         optional
  SSLCACertificateFile    "/etc/pki/katello/certs/katello-default-ca_e.crt"
  SSLVerifyDepth          3
  SSLOptions +StdEnvVars +ExportCertData
</VirtualHost>
""".strip()

HTTPD_CONF_WITHOUT_SSL = """
<VirtualHost *:80>
    ServerName a.b.c.com
</VirtualHost>
""".strip()

HTTPD_SSL_CONF_NO_VALUE = """
<VirtualHost *:443>
  ## SSL directives
  SSLEngine off
  SSLCertificateFile      ""
  SSLCertificateKeyFile   ""
  SSLCertificateChainFile ""
</VirtualHost>
""".strip()

NGINX_CONF = """
http {
    listen 80;
    listen 443;
    include /etc/nginx/conf.d/*.conf;
}
""".strip()

NGINX_SSL_CONF = """
server {
  ssl_certificate      "/a/b/c.rsa.crt";
  ssl_certificate_key   "/a/b/c.rsa.key";

  ssl_certificate     "/a/b/c.cecdsa.crt";
  ssl_certificate_key "/a/b/c.cecdsa.key";
}
""".strip()

NGINX_SSL_CONF_MULTIPLE_SERVERS = """
server {
    listen          443 ssl;
    server_name     www.example.com;
    ssl_certificate      "/a/b/www.example.com.crt";
    ssl_certificate_key   "/a/b/www.example.com.key";
    ssl_certificate     "/a/b/www.example.com.cecdsa.crt";
    ssl_certificate_key "/a/b/www.example.com.cecdsa.key";
}

server {
    listen          443 ssl;
    server_name     www.example.org;
    ssl_certificate      "/a/b/www.example.org.crt";
    ssl_certificate_key   "/a/b/www.example.org.key";
}
""".strip()

NGINX_CONF_WITHOUT_SSL = """
server {
  server_name 'a.b.c.com';
}
""".strip()

MSSQL_WITH_TLS = """
[sqlagent]
enabled = true
[EULA]
accepteula = Y
[memory]
memorylimitmb = 2048
[network]
tlscert = /tmp/mssql.pem
""".strip()

MSSQL_WITHOUT_TLS = """
[sqlagent]
enabled = true
[EULA]
accepteula = Y
[memory]
memorylimitmb = 2048
""".strip()

HTTPD_NSS_CERT_ENDATE = """
Not After : Mon Jan 18 07:02:43 2038
""".strip()

HTTPD_WITH_NSS = """
Listen 8443
<VirtualHost _default_:8443>
ServerName www.examplea.com:8443
NSSEngine on
# NSSCertificateDatabase old_path
NSSCertificateDatabase /etc/httpd/aliasa
# bellow is the NSSNickname configuration
NSSNickname testcertaê
</VirtualHost>
<VirtualHost :8443>
ServerName www.exampleb.com:8443
NSSEngine on
NSSCertificateDatabase /etc/httpd/aliasb
NSSNickname testcertb
</VirtualHost>
<VirtualHost :8443>
ServerName www.examplec.com:8443
NSSEngine off
NSSCertificateDatabase /etc/httpd/aliasc
NSSNickname testcertc
</VirtualHost>
""".strip()

HTTPD_WITH_NSS_OFF = """
Listen 8443
<VirtualHost _default_:8443>
NSSEngine off
NSSCertificateDatabase /etc/httpd/alias
NSSNickname testcert
</VirtualHost>
""".strip()

RSYSLOG_TLS_CFG1 = """
$ActionQueueFileName abc
$DefaultNetstreamDriver gtls
$DefaultNetstreamDriverCAFile /etc/pki/rsyslog/ca-cert.pem
$DefaultNetstreamDriverCertFile /etc/pki/rsyslog/sender-cert.pem
$DefaultNetstreamDriverKeyFile /etc/pki/rsyslog/sender-key.pem
$ActionSendStreamDriverAuthMode anon
"""

RSYSLOG_TLS_CFG2 = """
global(
workDirectory="/var/lib/rsyslog"
DefaultNetstreamDriverCAFile="/etc/rsyslog.d/certificates/crt/ca.crt"
DefaultNetstreamDriverCertFile="/etc/rsyslog.d/certificates/crt/cert.crt"
DefaultNetstreamDriverKeyFile="/etc/rsyslog.d/certificates/key/driver.key"
)
include(file="/etc/rsyslog.d/*.conf" mode="optional")
*.info;mail.none;authpriv.none;cron.none                /var/log/messages
"""

RSYSLOG_CFG3 = """
global(
workDirectory="/var/lib/rsyslog"
)
include(file="/etc/rsyslog.d/*.conf" mode="optional")
*.info;mail.none;authpriv.none;cron.none                /var/log/messages
"""

RSYSLOG_CLIENT_TLS_CA_CFG1 = """
global(
DefaultNetstreamDriverCAFile = "/usr/share/pki/ca-trust-source/anchors/ca.cer"
)
template(
type="string"
)
action(
type = "omfwd"
queue.type = "LinkedList"
)
"""

RSYSLOG_CLIENT_TLS_CA_CFG2 = """
authpriv.*                                              /var/log/secure
#action(type="omfwd"
#queue.maxdiskspace="1g"         # 1gb space limit (use as much as possible)
#queue.type="LinkedList"         # run asynchronously
# Remote Logging (we use TCP for reliable delivery)
#Target="remote_host" Port="XXX" Protocol="tcp")
global(DefaultNetstreamDriverCAFile="/opt/services/dlc/keystore/public.cert.pem")
auth,authpriv.* action(type="omfwd" protocol="tcp" target="abc.def.com" port="6514"
    StreamDriver="gtls" StreamDriverMode="1" StreamDriverAuthMode="anon")
"""

RSYSLOG_CLIENT_TLS_CA_CFG3 = """
$DefaultNetstreamDriverCAFile /usr/share/pki/ca-trust-source/anchors/ca.cer

template(
type="string"
)
action(
type = "omfwd"
queue.type = "LinkedList"
)
include(file="/etc/rsyslog.d/*.conf" mode="optional")
"""

RSYSLOG_CLIENT_TLS_CA_BAD_CFG3 = """
global(
DefaultNetstreamDriverCAFile  "/usr/share/pki/ca-trust-source/anchors/ca.cer"
)
template(
type="string"
)
action(
type = "omfwd"
queue.type = "LinkedList"
)
"""

RSYSLOG_CLIENT_TLS_CA_BAD_CFG4 = """
global(
DefaultNetstreamDriverCAFile =
)
template(
type="string"
)
action(
type = "omfwd"
queue.type = "LinkedList"
)
"""

RSYSLOG_CLIENT_TLS_CA_BAD_CFG5 = """
global(
DefaultNetstreamDriverCAFile
)
template(
type="string"
)
action(
type = "omfwd"
queue.type = "LinkedList"
)
"""

RSYSLOG_CLIENT_TLS_CA_BAD_CFG6 = """
template(
type="string"
)
action(
type = "omfwd"
queue.type = "LinkedList"
)
"""


@patch("os.path.exists", return_value=True)
@patch(builtin_open, new_callable=mock_open, read_data=HTTPD_CONF)
def test_httpd_certificate(m_open, m_exist):
    m_open.side_effect = [m_open.return_value, mock_open(read_data=HTTPD_SSL_CONF).return_value]
    broker = {
        httpd.httpd_configuration_files: ['/etc/httpd/conf/httpd.conf', '/etc/httpd/conf.d/ssl.conf']
    }
    result = httpd_ssl_certificate_files(broker)
    assert result == ['/etc/pki/katello/certs/gént-katello-apache.crt']

    m_open.side_effect = [m_open.return_value, mock_open(read_data=HTTPD_SSL_CONF_2).return_value]
    broker = {
        httpd.httpd_configuration_files: ['/etc/httpd/conf/httpd.conf', '/etc/httpd/conf.d/ssl.conf']
    }
    result = httpd_ssl_certificate_files(broker)
    # "/etc/pki/katello/certs/katello-apache_e.crt" not in the result
    assert result == ['/etc/pki/katello/certs/katello-apache.crt', '/etc/pki/katello/certs/katello-apache_d.crt']


def test_nginx_certificate():
    conf1 = context_wrap(NGINX_CONF, path='/etc/nginx/nginx.conf')
    conf2 = context_wrap(NGINX_SSL_CONF, path='/etc/nginx/conf.d/ssl.conf')

    broker = {
        Specs.nginx_conf: [conf1, conf2]
    }
    result = nginx_ssl_certificate_files(broker)
    assert result == ['/a/b/c.cecdsa.crt', '/a/b/c.rsa.crt']

    conf1 = context_wrap(NGINX_CONF, path='/etc/nginx/nginx.conf')
    conf2 = context_wrap(NGINX_SSL_CONF_MULTIPLE_SERVERS, path='/etc/nginx/conf.d/ssl.conf')

    broker = {
        Specs.nginx_conf: [conf1, conf2]
    }
    result = nginx_ssl_certificate_files(broker)
    assert result == ['/a/b/www.example.com.cecdsa.crt', '/a/b/www.example.com.crt', '/a/b/www.example.org.crt']


@patch("os.path.exists", return_value=True)
@patch(builtin_open, new_callable=mock_open, read_data=HTTPD_CONF)
def test_httpd_ssl_cert_exception(m_open, m_exists):
    m_open.side_effect = [m_open.return_value, mock_open(read_data=HTTPD_CONF_WITHOUT_SSL).return_value]
    broker1 = {
        httpd.httpd_configuration_files: ['/etc/httpd/conf/httpd.conf', '/etc/httpd/conf.d/no_ssl.conf']
    }
    with pytest.raises(SkipComponent):
        httpd_ssl_certificate_files(broker1)

    m_open.side_effect = [m_open.return_value, mock_open(read_data=HTTPD_SSL_CONF_NO_VALUE).return_value]
    broker2 = {
        httpd.httpd_configuration_files: ['/etc/httpd/conf/httpd.conf', '/etc/httpd/conf.d/no_ssl.conf']
    }
    with pytest.raises(SkipComponent):
        httpd_ssl_certificate_files(broker2)


def test_nginx_ssl_cert_exception():
    conf1 = context_wrap(NGINX_CONF, path='/etc/nginx/nginx.conf')
    conf2 = context_wrap(NGINX_CONF_WITHOUT_SSL, path='/etc/nginx/conf.d/no_ssl.conf')
    broker1 = {
        Specs.nginx_conf: [conf1, conf2]
    }
    with pytest.raises(SkipComponent):
        nginx_ssl_certificate_files(broker1)


def test_mssql_tls_cert_exception():
    conf1 = MsSQLConf(context_wrap(MSSQL_WITH_TLS, path='/var/opt/mssql/mssql.conf'))
    broker1 = {
        MsSQLConf: conf1
    }
    result = mssql_tls_cert_file(broker1)
    assert result == "/tmp/mssql.pem"


def test_mssql_tls_no_cert_exception():
    conf1 = MsSQLConf(context_wrap(MSSQL_WITHOUT_TLS, path='/var/opt/mssql/mssql.conf'))
    broker1 = {
        MsSQLConf: conf1
    }
    with pytest.raises(SkipComponent):
        mssql_tls_cert_file(broker1)


@patch("os.path.exists", return_value=True)
@patch(builtin_open, new_callable=mock_open, read_data=HTTPD_CONF)
def test_httpd_certificate_info_in_nss(m_open, m_exists):
    m_open.side_effect = [m_open.return_value, mock_open(read_data=HTTPD_WITH_NSS).return_value]
    broker = {
        httpd.httpd_configuration_files: ['/etc/httpd/conf/httpd.conf', '/etc/httpd/conf.d/nss.conf']
    }
    result = httpd_certificate_info_in_nss(broker)
    assert result == [('/etc/httpd/aliasa', 'testcertaê'), ('/etc/httpd/aliasb', 'testcertb')]


@patch("os.path.exists", return_value=True)
@patch(builtin_open, new_callable=mock_open, read_data=HTTPD_CONF)
def test_httpd_certificate_info_in_nss_exception(m_open, m_exists):
    m_open.side_effect = [m_open.return_value, mock_open(read_data=HTTPD_WITH_NSS_OFF).return_value]
    broker = {
        httpd.httpd_configuration_files: ['/etc/httpd/conf/httpd.conf', '/etc/httpd/conf.d/nss.conf']
    }
    with pytest.raises(SkipComponent):
        httpd_certificate_info_in_nss(broker)


def test_rsyslog_certification():
    rsys_par = RsyslogConf(context_wrap(RSYSLOG_TLS_CFG1, path='/etc/rsyslog.conf'))
    rsys_com = RsyslogAllConf([rsys_par])
    broker = {
        RsyslogAllConf: rsys_com
    }
    result = rsyslog_tls_cert_file(broker)
    assert result == '/etc/pki/rsyslog/sender-cert.pem'

    rsys_par = RsyslogConf(context_wrap(RSYSLOG_TLS_CFG2, path='/etc/rsyslog.conf'))
    rsys_com = RsyslogAllConf([rsys_par])
    broker = {
        RsyslogAllConf: rsys_com
    }
    result = rsyslog_tls_cert_file(broker)
    assert result == '/etc/rsyslog.d/certificates/crt/cert.crt'


def test_rsyslog_certification_exception():
    rsys_par = RsyslogConf(context_wrap(RSYSLOG_CFG3, path='/etc/rsyslog.conf'))
    rsys_com = RsyslogAllConf([rsys_par])
    broker = {
        RsyslogAllConf: rsys_com
    }
    with pytest.raises(SkipComponent):
        rsyslog_tls_cert_file(broker)


def test_rsyslog_ca_certification():
    rsys_par = RsyslogConf(context_wrap(RSYSLOG_CLIENT_TLS_CA_CFG1, path='/etc/rsyslog.conf'))
    rsys_com = RsyslogAllConf([rsys_par])
    broker = {
        RsyslogAllConf: rsys_com
    }
    result = rsyslog_tls_ca_cert_file(broker)
    assert result == '/usr/share/pki/ca-trust-source/anchors/ca.cer'

    rsys_par = RsyslogConf(context_wrap(RSYSLOG_CLIENT_TLS_CA_CFG2, path='/etc/rsyslog.conf'))
    rsys_com = RsyslogAllConf([rsys_par])
    broker = {
        RsyslogAllConf: rsys_com
    }
    result = rsyslog_tls_ca_cert_file(broker)
    assert result == '/opt/services/dlc/keystore/public.cert.pem'

    rsys_par = RsyslogConf(context_wrap(RSYSLOG_CLIENT_TLS_CA_CFG3, path='/etc/rsyslog.conf'))
    rsys_par2 = RsyslogConf(context_wrap(RSYSLOG_CLIENT_TLS_CA_BAD_CFG6, path='/etc/rsyslog.d/a.conf'))
    rsys_com = RsyslogAllConf([rsys_par, rsys_par2])
    broker = {
        RsyslogAllConf: rsys_com
    }
    result = rsyslog_tls_ca_cert_file(broker)
    assert result == '/usr/share/pki/ca-trust-source/anchors/ca.cer'


def test_rsyslog_ca_certification_exception():
    bad_example_confs = [
        RSYSLOG_CLIENT_TLS_CA_BAD_CFG3,
        RSYSLOG_CLIENT_TLS_CA_BAD_CFG4,
        RSYSLOG_CLIENT_TLS_CA_BAD_CFG5,
        RSYSLOG_CLIENT_TLS_CA_BAD_CFG6
    ]
    for bad_conf in bad_example_confs:
        rsys_par = RsyslogConf(context_wrap(bad_conf, path='/etc/rsyslog.conf'))
        rsys_com = RsyslogAllConf([rsys_par])
        broker = {
            RsyslogAllConf: rsys_com
        }
        with pytest.raises(SkipComponent):
            rsyslog_tls_ca_cert_file(broker)



import doctest
import pytest

from insights.core.dr import SkipComponent
from insights.parsers import nftables
from insights.parsers.nftables import NftListRuleSet
from insights.tests import context_wrap


NTT_LIST_RULESET_DOC = """
{"nftables": [{"metainfo": {"version": "0.9.3", "release_name": "Topsy", "json_schema_version": 1}}, {"table": {"family": "ip", "name": "table1", "handle": 17}}, {"map": {"family": "ip", "name": "example_map", "table": "table1", "type": "ipv4_addr", "handle": 2, "map": "verdict", "elem": [["192.0.2.1", {"accept": null}], ["192.0.2.2", {"drop": null}], ["192.0.2.3", {"accept": null}]]}}, {"set": {"family": "ip", "name": "example_set", "table": "table1", "type": "ipv4_addr", "handle": 4}}, {"chain": {"family": "ip", "table": "table1", "name": "chain1", "handle": 1, "type": "filter", "hook": "input", "prio": 0, "policy": "accept"}}, {"rule": {"family": "ip", "table": "table1", "chain": "chain1", "handle": 3, "expr": [{"vmap": {"key": {"payload": {"protocol": "ip", "field": "saddr"}}, "data": "@example_map"}}]}}, {"rule": {"family": "ip", "table": "table1", "chain": "chain1", "handle": 7, "expr": [{"match": {"op": "==", "left": {"payload": {"protocol": "tcp", "field": "dport"}}, "right": 22}}, {"counter": {"packets": 29, "bytes": 1976}}, {"accept": null}]}}]}
""".strip()

NTT_LIST_RULESET_OUTPU1 = """
{"nftables": [{"metainfo": {"version": "0.9.3", "release_name": "Topsy", "json_schema_version": 1}}, {"table": {"family": "ip", "name": "example_table", "handle": 17}}, {"map": {"family": "ip", "name": "example_map", "table": "example_table", "type": "ipv4_addr", "handle": 2, "map": "verdict", "elem": [["192.0.2.1", {"accept": null}], ["192.0.2.2", {"drop": null}], ["192.0.2.3", {"accept": null}]]}}, {"set": {"family": "ip", "name": "example_set", "table": "example_table", "type": "ipv4_addr", "handle": 4}}, {"chain": {"family": "ip", "table": "example_table", "name": "example_chain", "handle": 1, "type": "filter", "hook": "input", "prio": 0, "policy": "accept"}}, {"rule": {"family": "ip", "table": "example_table", "chain": "example_chain", "handle": 3, "expr": [{"vmap": {"key": {"payload": {"protocol": "ip", "field": "saddr"}}, "data": "@example_map"}}]}}, {"table": {"family": "ip", "name": "nftables_svc", "handle": 18}}, {"chain": {"family": "ip", "table": "nftables_svc", "name": "INPUT", "handle": 1, "type": "filter", "hook": "input", "prio": 0, "policy": "accept"}}, {"rule": {"family": "ip", "table": "nftables_svc", "chain": "INPUT", "handle": 2, "expr": [{"match": {"op": "==", "left": {"payload": {"protocol": "tcp", "field": "dport"}}, "right": 22}}, {"accept": null}]}}, {"rule": {"family": "ip", "table": "nftables_svc", "chain": "INPUT", "handle": 3, "expr": [{"match": {"op": "==", "left": {"payload": {"protocol": "tcp", "field": "dport"}}, "right": 443}}, {"accept": null}]}}, {"rule": {"family": "ip", "table": "nftables_svc", "chain": "INPUT", "handle": 4, "expr": [{"reject": null}]}}]}
""".strip()

NTT_LIST_RULESET_TWO_CHAINS = """
{"nftables": [{"metainfo": {"version": "0.9.3", "release_name": "Topsy", "json_schema_version": 1}}, {"table": {"family": "ip", "name": "example_table", "handle": 17}}, {"map": {"family": "ip", "name": "example_map", "table": "example_table", "type": "ipv4_addr", "handle": 2, "map": "verdict", "elem": [["192.0.2.1", {"accept": null}], ["192.0.2.2", {"drop": null}], ["192.0.2.3", {"accept": null}]]}}, {"set": {"family": "ip", "name": "example_set", "table": "example_table", "type": "ipv4_addr", "handle": 4}}, {"chain": {"family": "ip", "table": "example_table", "name": "example_chain", "handle": 1, "type": "filter", "hook": "input", "prio": 0, "policy": "accept"}}, {"rule": {"family": "ip", "table": "example_table", "chain": "example_chain", "handle": 3, "expr": [{"vmap": {"key": {"payload": {"protocol": "ip", "field": "saddr"}}, "data": "@example_map"}}]}}, {"rule": {"family": "ip", "table": "example_table", "chain": "example_chain", "handle": 7, "expr": [{"match": {"op": "==", "left": {"payload": {"protocol": "tcp", "field": "dport"}}, "right": 22}}, {"counter": {"packets": 808, "bytes": 61250}}, {"accept": null}]}}, {"chain": {"family": "ip", "table": "example_table", "name": "postrouting", "handle": 8, "type": "nat", "hook": "postrouting", "prio": 100, "policy": "accept"}}, {"rule": {"family": "ip", "table": "example_table", "chain": "postrouting", "handle": 9, "expr": [{"match": {"op": "==", "left": {"payload": {"protocol": "tcp", "field": "dport"}}, "right": 22}}, {"accept": null}]}}]}
""".strip()

EMPTY_OUTPUT = """
{"nftables": [{"metainfo": {"version": "0.9.3", "release_name": "Topsy", "json_schema_version": 1}}]}
""".strip()


def test_module_documentation():
    failed, _ = doctest.testmod(
        nftables,
        globs={
            "nft_obj": NftListRuleSet(context_wrap(NTT_LIST_RULESET_DOC))
        }
    )
    assert failed == 0


def test_nft_list_ruleset():
    nft_obj = NftListRuleSet(context_wrap(NTT_LIST_RULESET_OUTPU1))

    tables = nft_obj.tables('ip')
    assert len(tables) == 2
    assert 'example_table' in tables
    assert 'nftables_svc' in tables

    chains = nft_obj.chains('ip', 'example_table')
    assert len(chains) == 1
    assert chains[0]['name'].value == 'example_chain'

    assert chains[0]['type'].value == 'filter'
    assert chains[0]['hook'].value == 'input'
    assert chains[0]['prio'].value == 0
    assert chains[0]['policy'].value == 'accept'

    example_chain_rules = nft_obj.rules('ip', 'example_table', 'example_chain')
    assert len(example_chain_rules) == 1
    # {"rule": {"family": "ip", "table": "example_table", "chain": "example_chain", "handle": 3, "expr": [{"vmap": {"key": {"payload": {"protocol": "ip", "field": "saddr"}}, "data": "@example_map"}}]}}
    assert 'vmap' in example_chain_rules[0]['expr'][0]
    assert example_chain_rules[0]['expr'][0]['vmap']['key']['payload']['protocol'].value == 'ip'
    assert example_chain_rules[0]['expr'][0]['vmap']['key']['payload']['field'].value == 'saddr'
    assert example_chain_rules[0]['expr'][0]['vmap']['data'].value == '@example_map'

    svc_chains = nft_obj.chains('ip', 'nftables_svc')
    assert len(svc_chains) == 1
    assert 'INPUT' == svc_chains[0]['name'].value

    nftables_svc_input_rules = nft_obj.rules('ip', 'nftables_svc', 'INPUT')
    assert len(nftables_svc_input_rules) == 3
    # {"rule": {"family": "ip", "table": "nftables_svc", "chain": "INPUT", "handle": 2, "expr": [{"match": {"op": "==", "left": {"payload": {"protocol": "tcp", "field": "dport"}}, "right": 22}}, {"accept": null}]}}
    first_rule = nftables_svc_input_rules[0]
    assert first_rule['expr'][0]['match']['left']['payload']['protocol'].value == 'tcp'
    assert first_rule['expr'][0]['match']['left']['payload']['field'].value == 'dport'
    assert first_rule['expr'][0]['match']['op'].value == '=='
    assert first_rule['expr'][0]['match']['right'].value == 22
    assert first_rule['expr'][1]['accept'].value is None
    # {"rule": {"family": "ip", "table": "nftables_svc", "chain": "INPUT", "handle": 3, "expr": [{"match": {"op": "==", "left": {"payload": {"protocol": "tcp", "field": "dport"}}, "right": 443}}, {"accept": null}]}}
    second_rule = nftables_svc_input_rules[1]
    assert second_rule['expr'][0]['match']['left']['payload']['protocol'].value == 'tcp'
    assert second_rule['expr'][0]['match']['left']['payload']['field'].value == 'dport'
    assert second_rule['expr'][0]['match']['op'].value == '=='
    assert second_rule['expr'][0]['match']['right'].value == 443
    assert second_rule['expr'][1]['accept'].value is None
    # {"rule": {"family": "ip", "table": "nftables_svc", "chain": "INPUT", "handle": 4, "expr": [{"reject": null}]}}
    third_rule = nftables_svc_input_rules[2]
    assert third_rule['expr'][0]['reject'].value is None

    # test map info in data
    # {"map": {"family": "ip", "name": "example_map", "table": "example_table", "type": "ipv4_addr", "handle": 2, "map": "verdict", "elem": [["192.0.2.1", {"accept": null}], ["192.0.2.2", {"drop": null}], ["192.0.2.3", {"accept": null}]]}}
    assert 'map' in nft_obj.data['nftables'][2]
    map_info = nft_obj.data['nftables'][2]['map']
    assert map_info['name'].value == 'example_map'
    assert map_info['table'].value == 'example_table'
    assert map_info['type'].value == 'ipv4_addr'
    assert map_info['map'].value == 'verdict'
    assert len(map_info['elem']) == 1
    map_ele = map_info['elem'][0]
    # "['192.0.2.1', {'accept': None}] ['192.0.2.2', {'drop': None}] ['192.0.2.3', {'accept': None}]"
    assert '192.0.2.1' in map_ele.value
    assert '192.0.2.2' in map_ele.value
    assert '192.0.2.3' in map_ele.value

    # test set info in data
    # {"set": {"family": "ip", "name": "example_set", "table": "example_table", "type": "ipv4_addr", "handle": 4}}
    assert 'set' in nft_obj.data['nftables'][3]
    set_info = nft_obj.data['nftables'][3]['set']
    assert set_info['name'].value == 'example_set'
    assert set_info['table'].value == 'example_table'
    assert set_info['type'].value == 'ipv4_addr'

    nft2_obj = NftListRuleSet(context_wrap(NTT_LIST_RULESET_TWO_CHAINS))
    chains = nft2_obj.chains('ip', 'example_table')
    assert len(chains) == 2
    assert 'postrouting' in [item['name'].value for item in chains]
    rules = nft2_obj.rules('ip', 'example_table', 'postrouting')
    assert len(rules) == 1
    first_rule = rules[0]
    # {"rule": {"family": "ip", "table": "example_table", "chain": "postrouting", "handle": 9, "expr": [{"match": {"op": "==", "left": {"payload": {"protocol": "tcp", "field": "dport"}}, "right": 22}}, {"accept": null}]}}
    assert first_rule['expr'][0]['match']['left']['payload']['protocol'].value == 'tcp'
    assert first_rule['expr'][0]['match']['left']['payload']['field'].value == 'dport'
    assert first_rule['expr'][0]['match']['op'].value == '=='
    assert first_rule['expr'][0]['match']['right'].value == 22
    assert first_rule['expr'][1]['accept'].value is None

    assert not nft2_obj.tables('non_exist_ip')
    assert not nft2_obj.chains('ip', 'non_exist_table')
    assert not nft2_obj.rules('ip', 'example_table', 'non_exist_chain')


def test_except_in_nft_list_ruleset():
    with pytest.raises(SkipComponent):
        NftListRuleSet(context_wrap(EMPTY_OUTPUT))



"""
Custom datasources for the files under /sys/fs/cgroup/memory
"""
import os

from insights.core.context import HostContext
from insights.core.dr import SkipComponent
from insights.core.plugins import datasource
from insights.core.spec_factory import DatasourceProvider


@datasource(HostContext)
def sys_fs_cgroup_uniq_memory_swappiness(broker):
    """
    This datasource reads all the memory.swappiness files under /sys/fs/cgroup/memory directory
    one by one and returns the uniq memory swappiness setting of the all control groups.

    The output of this datasource looks like:
        10   1
        60   66

    Returns:
        str: Returns a multiline string in the format as ``value   count``.

    Raises:
        SkipComponent: When any exception occurs.
    """
    file_name = "memory.swappiness"
    data = {}

    for root, _, files in os.walk("/sys/fs/cgroup/memory"):
        if file_name in files:
            file_path = os.path.join(root, file_name)
            with open(file_path, 'r') as setting:
                key = setting.read().strip()
                data[key] = data.get(key, 0) + 1

    if not data:
        raise SkipComponent()

    data_list = []
    for value, count in data.items():
        data_list.append("{0}   {1}".format(value, count))

    return DatasourceProvider(content="\n".join(data_list), relative_path='insights_commands/sys_fs_cgroup_uniq_memory_swappiness')



"""
IronicInspectorLog - file ``/var/log/ironic-inspector/ironic-inspector.log``
============================================================================

This is a standard log parser based on the LogFileOutput class.

Sample input::

    2018-12-05 17:20:41.404 25139 ERROR requests.packages.urllib3.connection [-] Certificate did not match expected hostname: 10.xx.xx.xx. Certificate: {'subjectAltName': (('DNS', '10.xx.xx.xx'),), 'notBefore': u'Dec  4 12:02:36 2018 GMT', 'serialNumber': u'616460101648CCDF5727C', 'notAfter': 'Jun 21 21:40:11 2019 GMT', 'version': 3L, 'subject': ((('commonName', u'10.xx.xx.xx'),),), 'issuer': ((('commonName', u'Local Signing Authority'),), (('commonName', u'616460a1-da41448c-cdf566ff'),))}

Examples:

    >>> assert len(log.lines) == 1
    >>> assert log.lines[0] == "2018-12-05 17:20:41.404 25139 ERROR requests.packages.urllib3.connection [-] Certificate did not match expected hostname: 10.xx.xx.xx. Certificate: {'subjectAltName': (('DNS', '10.xx.xx.xx'),), 'notBefore': u'Dec  4 12:02:36 2018 GMT', 'serialNumber': u'616460101648CCDF5727C', 'notAfter': 'Jun 21 21:40:11 2019 GMT', 'version': 3L, 'subject': ((('commonName', u'10.xx.xx.xx'),),), 'issuer': ((('commonName', u'Local Signing Authority'),), (('commonName', u'616460a1-da41448c-cdf566ff'),))}"
"""
from insights.specs import Specs
from .. import LogFileOutput, parser


@parser(Specs.ironic_inspector_log)
class IronicInspectorLog(LogFileOutput):
    """Provide access to Ironic Inspector logs using the base class ``LogFileOutput``.

    .. note::
        Please refer to the super-class :class:`insights.core.LogFileOutput`
    """
